package org.szu;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

import net.automatalib.words.Alphabet;
import net.automatalib.words.Word;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.mnist.dfa.symbols.LengthSymbol;
import org.szu.mnist.dfa.symbols.MatrixSymbol;
import org.szu.mnist.dfa.symbols.PositionSymbol;
import org.szu.mnist.mapper.*;
import org.szu.mnist.sul.*;


public final class DataAnalyzer {

    private static Logger log = LoggerFactory.getLogger(DataAnalyzer.class);

    //numberkind
    private static int numKind = 1; //[0,9]
    //Symbol kinds
    private static int symKind = 1;
    private static int wordKind = 0;

    //format
    private static int rowOut = 28;
    private static int colOut = 14;
    private static int rowSpan = 28;
    private static int colSpan = 14;
    private static int valueScale = 2;

    //beta
    private static float delta = 0.25f;



    private DataAnalyzer() {
        // prevent instantiation
    }

    protected static void analysis0(AlphaBeta<MatrixSymbol> mapper, boolean train) {
        try {
            String fileName_wp = "Words_" + numKind + "_" + rowOut + "_" + colOut + "_" + rowSpan + "_" + colSpan + "_" + valueScale + "_" + train + "_P.txt";
            String fileName_wn = "Words_" + numKind + "_" + rowOut + "_" + colOut + "_" + rowSpan + "_" + colSpan + "_" + valueScale + "_" + train + "_N.txt";
            String fileName_dp = "Data_" + numKind + "_" + rowOut + "_" + colOut + "_" + rowSpan + "_" + colSpan + "_" + valueScale + "_" + train + "_P.txt";
            String fileName_dn = "Data_" + numKind + "_" + rowOut + "_" + colOut + "_" + rowSpan + "_" + colSpan + "_" + valueScale + "_" + train + "_N.txt";
            FileOutputStream fos_wp = new FileOutputStream(fileName_wp);
            PrintWriter out_wp = new PrintWriter(fos_wp);
            FileOutputStream fos_wn = new FileOutputStream(fileName_wn);
            PrintWriter out_wn = new PrintWriter(fos_wn);
            FileOutputStream fos_dp = new FileOutputStream(fileName_dp);
            PrintWriter out_dp = new PrintWriter(fos_dp);
            FileOutputStream fos_dn = new FileOutputStream(fileName_dn);
            PrintWriter out_dn = new PrintWriter(fos_dn);
            log.info("Get the input alphabet of system under learning ...");
            Alphabet<MatrixSymbol> inputs = mapper.getAlphabetFromData();
            //log.info("Alphabet Size from data: " + inputs.size());
            //log.info("Alphabet Size: " + mapper.getAlphabetSize());
            Map<Word<MatrixSymbol>, List<DataClass>> wordSources;
            if(train) wordSources = mapper.getWordTrainSources();
            else wordSources = mapper.getWordTestSources();

            List<Word<MatrixSymbol>> pkeys = new ArrayList<>() ;
            List<Word<MatrixSymbol>> nkeys = new ArrayList<>() ;
            int pds, nds;
            int pn = 0;
            int nn = 0;
            for (Word<MatrixSymbol> key : wordSources.keySet()) {
                pds = 0; nds = 0;
                for(DataClass d : wordSources.get(key)) {
                    if(d.label) pds++;
                    else nds++;
                }
                if(pds >= nds) {
                    pkeys.add(key);
                    out_dp.println(pn + " " + pds + " " + nds);
                    pn++;
                }
                else {
                    nkeys.add(key);
                    out_dn.println(nn + " " + pds + " " + nds);
                    nn++;
                }
            }
            out_wp.println(pkeys.size() + " " + inputs.size());
            int n =0;
            for (Word<MatrixSymbol> key : pkeys) {
                out_wp.println(n + " " + key.length() + " " + key.toString());
                n++;
            }
            out_wn.println(nkeys.size() + " " + inputs.size());
            n =0;
            for (Word<MatrixSymbol> key : nkeys) {
                out_wn.println(n + " " + key.length() + " " + key.toString());
                n++;
            }
            System.out.println("Proceeds " + pkeys.size() + " positive words and " + nkeys.size() + " negative words in " + train + " dataset");
            out_wp.close(); out_wn.close(); out_dp.close(); out_dn.close();
            fos_wp.close(); fos_wn.close(); fos_dp.close(); fos_dn.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected static void analysis(AlphaBeta<MatrixSymbol> mapper, boolean train) {
        try {
            String fileName_w = "Words_" + numKind + "_" + rowOut + "_" + colOut + "_" + rowSpan + "_" + colSpan + "_" + valueScale + "_" + train + ".txt";
            String fileName_d = "Data_" + numKind + "_" + rowOut + "_" + colOut + "_" + rowSpan + "_" + colSpan + "_" + valueScale + "_" + train + ".txt";

            FileOutputStream fos_w = new FileOutputStream(fileName_w);
            PrintWriter out_w = new PrintWriter(fos_w);
            FileOutputStream fos_d = new FileOutputStream(fileName_d);
            PrintWriter out_d = new PrintWriter(fos_d);

            log.info("Get the input alphabet of system under learning ...");
            Alphabet<MatrixSymbol> inputs = mapper.getAlphabetFromData();
            //log.info("Alphabet Size from data: " + inputs.size());
            //log.info("Alphabet Size: " + mapper.getAlphabetSize());
            Map<Word<MatrixSymbol>, List<DataClass>> wordSources;
            if(train) wordSources = mapper.getWordTrainSources();
            else wordSources = mapper.getWordTestSources();

            out_w.println(wordSources.keySet().size() + " " + inputs.size());
            out_d.println(wordSources.keySet().size());
            int pds, nds;
            int label;
            for (Word<MatrixSymbol> key : wordSources.keySet()) {
                pds = 0; nds = 0;
                for(DataClass d : wordSources.get(key)) {
                    if(d.label) pds++;
                    else nds++;
                }
                if(pds >= nds) label = 1;
                else label = 0;
                out_w.println(label + " " + key.length() + " " + key.toString());
                out_d.println(pds + " " + nds);
            }

            System.out.println("Proceeds " + wordSources.keySet().size() + " words in " + train + " dataset");
            out_w.close(); out_d.close();
            fos_w.close(); fos_d.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public static void main(String[] args) throws IOException {
        log.info("Analyzing dataset via num " + numKind);
        DataProvider.setKind(numKind);
        log.info("Loading system under learning ...");
        //Model sulModel = new MLPMnistSingleLayerModel();
        Model sulModel = new MnistClassifierModel();
        int[] paras_row = {28, 28, 14, 28,  7, 28,  4, 28,  2, 28,  1};
        int[] paras_col = {28, 14, 28,  7, 28,  4, 28,  2, 28,  1, 28};
        int len = paras_row.length;  //11;
        for(int i = 0; i < len; i++) {
            rowOut = paras_row[i];
            rowSpan = rowOut;
            colOut = paras_col[i];
            colSpan = colOut;
            log.info("=======================" + rowOut + "," + colOut + "===========================");
            log.info("Building abstraction ...");

            switch (wordKind) {
                case 0:
                    AlphaBeta<MatrixSymbol> mapper;
                    switch (symKind) {
                        case 0:
                            mapper = new MatrixAlphaBeta(sulModel);
                            break;
                        case 1:
                            mapper = new SumAlphaBeta(sulModel);
                            break;
                        case 2:
                            mapper = new MeanAlphaBeta(sulModel);
                            break;
                        default:
                            mapper = new MatrixAlphaBeta(sulModel);
                            break;
                    }
                    mapper.setOutSpan(rowSpan, colSpan);
                    mapper.setOutSize(rowOut, colOut);
                    mapper.setValueScale(valueScale);
                    //mapper.setDelta(delta);
                    //
                    analysis(mapper, true);
                    analysis(mapper, false);
                    break;

                case 1:
                    AlphaBeta<PositionSymbol> mapper1;
                    switch (symKind) {
                        case 0:
                            mapper1 = new PositionMatrixAlphaBeta(sulModel);
                            break;
                        case 1:
                            mapper1 = new PositionSumAlphaBeta(sulModel);
                            break;
                        case 2:
                            mapper1 = new PositionMeanAlphaBeta(sulModel);
                            break;
                        default:
                            mapper1 = new PositionMatrixAlphaBeta(sulModel);
                            break;
                    }
                    mapper1.setOutSpan(rowSpan, colSpan);
                    mapper1.setOutSize(rowOut, colOut);
                    mapper1.setValueScale(valueScale);
                    //mapper1.setDelta(delta);
                    // input
                    log.info("Get the input alphabet of system under learning ...");
                    Alphabet<PositionSymbol> inputs1 = mapper1.getAlphabetFromData();
                    log.info("Alphabet Size from data: " + inputs1.size());
                    log.info("Alphabet Size: " + mapper1.getAlphabetSize());
                    break;

                case 2:
                    AlphaBeta<LengthSymbol> mapper2;
                    switch (symKind) {
                        case 0:
                            mapper2 = new RLEMatrixAlphaBeta(sulModel);
                            break;
                        case 1:
                            mapper2 = new RLESumAlphaBeta(sulModel);
                            break;
                        case 2:
                            mapper2 = new RLEMeanAlphaBeta(sulModel);
                            break;
                        default:
                            mapper2 = new RLEMatrixAlphaBeta(sulModel);
                            break;
                    }
                    mapper2.setOutSpan(rowSpan, colSpan);
                    mapper2.setOutSize(rowOut, colOut);
                    mapper2.setValueScale(valueScale);
                    //mapper2.setDelta(delta);
                    // input
                    log.info("Get the input alphabet of system under learning ...");
                    Alphabet<LengthSymbol> inputs2 = mapper2.getAlphabetFromData();
                    log.info("Alphabet Size from data: " + inputs2.size());
                    log.info("Alphabet Size: " + mapper2.getAlphabetSize());
                    break;

                default:

                    break;
            }
        }


    }


}
