package org.szu;

import org.apache.commons.io.FilenameUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.mnist.DFALearner;
import org.szu.mnist.dfa.symbols.LengthSymbol;
import org.szu.mnist.dfa.symbols.MatrixSymbol;
import org.szu.mnist.dfa.symbols.PositionSymbol;
import org.szu.mnist.mapper.*;
import org.szu.mnist.sul.DataProvider;
import org.szu.mnist.sul.MnistClassifierModel;
import org.szu.mnist.sul.Model;
import org.szu.mnist.sul.Models4MNIST;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

public class Learner {
    private static Logger log = LoggerFactory.getLogger(Learner.class);
    private static  String MNIST_ROOT = FilenameUtils.concat(System.getProperty("user.home"),"MNIST");

    //numberkind
    private static int numKind = 1; //[0,9]
    //Symbol kinds
    private static int symKind = 1;
    private static int wordKind = 0;
    //algorithm kinds
    private static int algoKind = 0;
    //network layers
    private static int numLayers = 0; //1,2,10

    //Alphabet from data
    private static boolean withDataAlphabet = true;
    //format
    private static boolean withFormat = true;
    private static int rowOut = 1;
    private static int colOut = 28;
    private static int rowSpan = 1;
    private static int colSpan = 28;
    private static int valueScale = 2;
    //beta
    private static float delta = 0.25f;
    //
    private static boolean withCache = true;
    private static boolean withTest = true;

    //membership query
    private static boolean withWord = false; //
    //limited number for a list of a word
    private static int listLimited = 99;
    //equivalence query
    private static boolean withPassiveE = true; //
    //equivalence query with training set
    private static int testRatio = 10; //
    //Passive DFA
    private static int pRatio = 5; // [0,10]
    private static int nRatio = 5; // [0,10]
    //equivalence query with passive DFA
    private static int eqKind = 0; //7
    private static int bound = 1000;
    private static int dataTime = 1;
    private static int maxRefined = 10000;

    private static boolean savingDFA = true;

    private static boolean isPassive = false;

    private static boolean isLoop = false;


    public static void main(String[] args) throws Exception {
        String str = parseAdditionalOptions(args);
        log.info(str);

        if(!isPassive && !isLoop) {
            log.info("Analyzing dataset via num " + numKind);
            DataProvider.setKind(numKind);
            Models4MNIST.setNumLayers(numLayers);
            log.info("Loading system under learning ...");
            Model sulModel = new Models4MNIST();
            log.info("=======================Row: " + rowOut + ", Col: " + colOut + "===========================");
            //result file
            StringBuffer buffer = new StringBuffer();
            SimpleDateFormat ft = new SimpleDateFormat("-yyMMdd-HHmmss");
            Date date = new Date();
            String sDate = ft.format(date);
            buffer.append("LearnedDFA");
            buffer.append("_");
            buffer.append(symKind);
            buffer.append("_");
            buffer.append(wordKind);
            buffer.append("_");
            buffer.append(rowOut);
            buffer.append("_");
            buffer.append(colOut);
            buffer.append("_");
            buffer.append(rowSpan);
            buffer.append("_");
            buffer.append(colSpan);
            buffer.append("_");
            buffer.append(valueScale);
            buffer.append("_");
            buffer.append(numKind);
            //start here
            //buffer.append("_");
            //buffer.append(pRatio);
            //buffer.append("_");
            //buffer.append(nRatio);
            //end here
            buffer.append("_");
            buffer.append(numLayers);
            buffer.append(sDate);
            buffer.append(".txt");
            String fileName = buffer.toString();
            String curPath = System.getProperty("user.dir") + File.separator + "results";
            File myPath = new File(curPath);
            if (!myPath.exists()) {
                myPath.mkdir();
                System.out.println(">>>>>>>>>>>>>>>>>>Created dir: " + curPath);
            }
            String fullName = curPath + File.separator + fileName;
            switch (wordKind) {
                case 0:
                    learnerByMatrix(sulModel, fullName);
                    break;
                case 1:
                    learnerByPosition(sulModel, fullName);
                    break;
                case 2:
                    learnerByLength(sulModel, fullName);
                    break;
                default:
                    System.out.println("Thanks for using DFA4MNIST!");
            }
            return;
        }

        if (isPassive) {
            //for (int k = 9; k >= 0; k--) {
            numKind = 1;
            int[] paras_row = {28, 28, 14, 28, 7, 28, 4, 28, 2, 28, 1};
            int[] paras_col = {28, 14, 28, 7, 28, 4, 28, 2, 28, 1, 28};
            int len = 11;
            for (int n = 0; n <= 10; n = n + 1) {
                nRatio = n;
                for (int p = 0; p <= 10; p = p + 1) {
                    pRatio = p;
                    for (int al = 0; al < 3; al++) {
                        algoKind = al;
                        for (int i = 7; i < len; i++) {
                            rowOut = paras_row[i];
                            rowSpan = rowOut;
                            colOut = paras_col[i];
                            colSpan = colOut;
                            log.info("=======================" + rowOut + "," + colOut + "===========================");
                            PassiveLearner passiveLearner = new PassiveLearner(rowOut, colOut, rowSpan, colSpan, valueScale);
                            passiveLearner.setNumKind(numKind);
                            passiveLearner.setAlgoKind(algoKind);
                            passiveLearner.setnRatio(nRatio);
                            passiveLearner.setpRatio(pRatio);
                            passiveLearner.learning();
                        }
                    }
                }
            }
            return;
        }

        for (int k = 1; k >= 1; k--) { //only 1
            numKind = k;
            log.info("Analyzing dataset via num " + numKind);
            DataProvider.setKind(numKind);
            int[] paras_layers = {1, 2, 10, 0};
            for (int li = 3; li >= 0; li--) {
                numLayers = paras_layers[li];
                log.info("Set the number of layers as " + numLayers);
                Models4MNIST.setNumLayers(numLayers);
                log.info("Loading system under learning ...");
                Model sulModel = new Models4MNIST();//MnistClassifierModel();
                //int[] paras_row = {28, 28, 14, 28, 7, 28, 4, 28, 2, 28, 1};
                //int[] paras_col = {28, 14, 28, 7, 28, 4, 28, 2, 28, 1, 28};
                //int len = 11;

                int[] paras_row = {28, 28, 2, 28, 1};
                int[] paras_col = {28, 2, 28, 1, 28};
                int len = 5;


                for (int n = 0; n <= 10; n = n + 5) {
                    nRatio = n;
                    for (int p = 0; p <= 10; p = p + 5) {
                        pRatio = p;
                        log.info("=======================pRatio: " + pRatio + ", nRatio: " + nRatio + "===========================");
                        for (int i = 0; i < len; i++) {
                            rowOut = paras_row[i];
                            rowSpan = rowOut;
                            colOut = paras_col[i];
                            colSpan = colOut;
                            log.info("=======================Row: " + rowOut + ", Col: " + colOut + "===========================");

                            //result file
                            StringBuffer buffer = new StringBuffer();
                            //SimpleDateFormat ft = new SimpleDateFormat("-yyMMdd-HHmmss");
                            //Date date = new Date();
                            //String sDate = ft.format(date);
                            buffer.append("LearnedDFA");
                            buffer.append("_");
                            buffer.append(symKind);
                            buffer.append("_");
                            buffer.append(wordKind);
                            buffer.append("_");
                            buffer.append(rowOut);
                            buffer.append("_");
                            buffer.append(colOut);
                            buffer.append("_");
                            buffer.append(rowSpan);
                            buffer.append("_");
                            buffer.append(colSpan);
                            buffer.append("_");
                            buffer.append(valueScale);
                            buffer.append("_");
                            buffer.append(numKind);
                            //start here
                            buffer.append("_");
                            buffer.append(pRatio);
                            buffer.append("_");
                            buffer.append(nRatio);
                            //end here
                            buffer.append("_");
                            buffer.append(numLayers);
                            //buffer.append(sDate);
                            buffer.append(".txt");
                            String fileName = buffer.toString();
                            String curPath = System.getProperty("user.dir") + File.separator + "results";
                            //start file existing?
                            File myPath = new File(curPath);
                            if (!myPath.exists()) {
                                myPath.mkdir();
                                System.out.println(">>>>>>>>>>>>>>>>>>Created dir: " + curPath);
                            }
                            boolean done = false;
                            File[] files = myPath.listFiles();
                            if (files != null) {
                                //for (int fi = 0; fi < files.length; fi++) {
                                //    File f = files[fi];
                                for (File f : files) {
                                    if (f.getName().equalsIgnoreCase(fileName)) {
                                        System.out.println(">>>>>>>>>>>>>>>>>>File existed: " + f.getName());
                                        done = true;
                                        break;
                                    }
                                }
                            }
                            //end file existing
                            if (!done) {
                                String fullName = curPath + File.separator + fileName;
                                switch (wordKind) {
                                    case 0:
                                        learnerByMatrix(sulModel, fullName);
                                        break;
                                    case 1:
                                        learnerByPosition(sulModel, fullName);
                                        break;
                                    case 2:
                                        learnerByLength(sulModel, fullName);
                                        break;
                                    default:
                                        System.out.println("Thanks for using DFA4MNIST!");
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private static String parseAdditionalOptions(String[] args) {
        int i = 0;
        StringBuffer buffer = new StringBuffer();
        while (i < args.length) {
            if (args[i].equalsIgnoreCase("-sym")) {
                int value = Integer.valueOf(args[i+1]);
                if(0 <= value || value <=2) {
                    symKind = value;
                    buffer.append("_symKind_"+value);
                }
                i=i+2;
            }
            else if (args[i].equalsIgnoreCase("-word")) {
                int value = Integer.valueOf(args[i+1]);
                if(0 <= value || value <=2) {
                    wordKind = value;
                    buffer.append("_wordKind_"+value);
                }
                i=i+2;
            }
            else if (args[i].equalsIgnoreCase("-algo")) {
                int value = Integer.valueOf(args[i+1]);
                if(0 <= value || value <=4) {
                    algoKind = value;
                    buffer.append("_algoKind_"+value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-or")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0 && value <= DataProvider.numRows) {
                    rowOut = value;
                    buffer.append("_rowOut_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-oc")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0 && value <= DataProvider.numColumns) {
                    colOut = value;
                    buffer.append("_colOut_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-sr")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0 && value <= DataProvider.numRows) {
                    rowSpan = value;
                    buffer.append("_rowSpan_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-sc")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0 && value <= DataProvider.numColumns) {
                    colSpan = value;
                    buffer.append("_colSpan_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-vs")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0) {
                    valueScale = value;
                    buffer.append("_valueScale_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-d")) {
                withDataAlphabet = true;
                i++;
                buffer.append("_dataAlphabet");
            }
            else if(args[i].equalsIgnoreCase("-nd")) {
                withDataAlphabet = false;
                i++;
                buffer.append("_non-dataAlphabet");
            }
            else if(args[i].equalsIgnoreCase("-f")) {
                withFormat = true;
                i=i+1;
                buffer.append("_strictFormat");
            }
            else if(args[i].equalsIgnoreCase("-nf")) {
                withFormat = false;
                i=i+1;
                buffer.append("_non-strictFormat");
            }
            else if(args[i].equalsIgnoreCase("-t")) {
                withTest = true;
                i=i+1;
                buffer.append("_withTest");
            }
            else if(args[i].equalsIgnoreCase("-nt")) {
                withTest = false;
                i=i+1;
                buffer.append("_non-Test");
            }
            else if(args[i].equalsIgnoreCase("-c")) {
                withCache = true;
                i=i+1;
                buffer.append("_withCache");
            }
            else if(args[i].equalsIgnoreCase("-nc")) {
                withCache = false;
                i=i+1;
                buffer.append("_non-Cache");
            }
            else if(args[i].equalsIgnoreCase("-nwd")) {
                withWord = false;
                i=i+1;
                buffer.append("_non-WordMembership");
            }
            else if(args[i].equalsIgnoreCase("-wd")) {
                withWord = true;
                i=i+1;
                buffer.append("_withWordMembership");
            }
            else if(args[i].equalsIgnoreCase("-limit")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0 ) {//&& value < 11
                    listLimited = value;
                    buffer.append("_listLimited_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-pe")) {
                withPassiveE = true;
                i=i+1;
                buffer.append("_withPassiveEquivalence");
            }
            else if(args[i].equalsIgnoreCase("-npe")) {
                withPassiveE = false;
                i=i+1;
                buffer.append("_non-PassiveEquivalence");
            }
            else if(args[i].equalsIgnoreCase("-bound")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0) {
                    bound = value;
                    buffer.append("_bound_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-pratio")) {
                int value = Integer.valueOf(args[i+1]);
                if(value >= 0 ) {//&& value < 11
                    pRatio = value;
                    buffer.append("_pRatio_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-nratio")) {
                int value = Integer.valueOf(args[i+1]);
                if(value >= 0 ) {//&& value < 11
                    nRatio = value;
                    buffer.append("_nRatio_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-eqkind")) {
                int value = Integer.valueOf(args[i+1]);
                if(value >= 0 && value < 7) {
                    eqKind = value;
                    buffer.append("_eqkind_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-dt")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0 ) {//&& value < 11
                    dataTime = value;
                    buffer.append("_dataTime_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-maxrf")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0 ) {//&& value < 11
                    maxRefined = value;
                    buffer.append("_maxRefine_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-tratio")) {
                int value = Integer.valueOf(args[i+1]);
                if(value > 0 ) {//&& value < 11
                    testRatio = value;
                    buffer.append("_testRatio_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-num")) {
                int value = Integer.valueOf(args[i+1]);
                if(value >= 0 && value < 10) {
                    numKind = value;
                    buffer.append("_numKind_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-save")) {
                savingDFA = true;
                i=i+1;
                buffer.append("_SavingDFA");
            }
            else if(args[i].equalsIgnoreCase("-nsave")) {
                savingDFA = false;
                i = i + 1;
                buffer.append("_non-SavingDFA");
            }
            else if(args[i].equalsIgnoreCase("-passive")) {
                isPassive = true;
                i = i + 1;
                buffer.append("_passive-learning");
            }
            else if(args[i].equalsIgnoreCase("-layer")) {
                int value = Integer.valueOf(args[i+1]);
                if(value >= 0) {
                    numLayers = value;
                    buffer.append("_layer_" + value);
                }
                i=i+2;
            }
            else if(args[i].equalsIgnoreCase("-loop")) {
                isLoop = true;
                i = i + 1;
                buffer.append("_looping");
            }
            else i++;
        }

        return buffer.toString();
    }

    private static void learnerByMatrix(Model sulModel, String fileName) {
        log.info("Building abstraction ...");
        AlphaBeta<MatrixSymbol> mapper;
        switch (symKind) {
            case 0:
                mapper = new MatrixAlphaBeta(sulModel);
                break;
            case 1:
                mapper = new SumAlphaBeta(sulModel);
                break;
            case 2:
                mapper = new MeanAlphaBeta(sulModel);
                break;
            default:
                mapper = new SumAlphaBeta(sulModel);
                break;
        }
        mapper.setWithFormat(withFormat);
        mapper.setOutSpan(rowSpan, colSpan);
        mapper.setOutSize(rowOut, colOut);
        mapper.setValueScale(valueScale);
        mapper.setDelta(delta);
        mapper.setListLimited(listLimited);

        DFALearner<MatrixSymbol> dfaLearner = new DFALearner<>(sulModel, mapper, fileName);
        //alphabet
        dfaLearner.setWithDataAlphabet(withDataAlphabet);
        //algorithm
        dfaLearner.setAlgoKind(algoKind);
        //simulator
        dfaLearner.setpRatio(pRatio);
        dfaLearner.setnRatio(nRatio);
        //membership
        dfaLearner.setWithWord(withWord);
        dfaLearner.setListLimited(listLimited);
        dfaLearner.setWithCache(withCache);
        //equivalence
        dfaLearner.setWithPassiveE(withPassiveE);
        dfaLearner.setTestRatio(testRatio);
        dfaLearner.setEqKind(eqKind);
        dfaLearner.setBound(bound);
        dfaLearner.setMaxRefined(maxRefined);
        dfaLearner.setDataTime(dataTime);
        //test
        dfaLearner.setWithTest(withTest);
        //saving
        dfaLearner.setSavingDFA(savingDFA);
        //running
        dfaLearner.learning();
    }

    private static void learnerByPosition(Model sulModel, String fileName) {
        log.info("Building abstraction ...");
        AlphaBeta<PositionSymbol> mapper;
        switch (symKind) {
            case 0:
                mapper = new PositionMatrixAlphaBeta(sulModel);
                break;
            case 1:
                mapper = new PositionSumAlphaBeta(sulModel);
                break;
            case 2:
                mapper = new PositionMeanAlphaBeta(sulModel);
                break;
            default:
                mapper = new PositionSumAlphaBeta(sulModel);
                break;
        }
        mapper.setWithFormat(withFormat);
        mapper.setOutSpan(rowSpan, colSpan);
        mapper.setOutSize(rowOut, colOut);
        mapper.setValueScale(valueScale);
        mapper.setDelta(delta);
        mapper.setListLimited(listLimited);

        DFALearner<PositionSymbol> dfaLearner = new DFALearner<>(sulModel, mapper, fileName);
        //alphabet
        dfaLearner.setWithDataAlphabet(withDataAlphabet);
        //algorithm
        dfaLearner.setAlgoKind(algoKind);
        //simulator
        dfaLearner.setpRatio(pRatio);
        dfaLearner.setnRatio(nRatio);
        //membership
        dfaLearner.setWithWord(withWord);
        dfaLearner.setListLimited(listLimited);
        dfaLearner.setWithCache(withCache);
        //equivalence
        dfaLearner.setWithPassiveE(withPassiveE);
        dfaLearner.setTestRatio(testRatio);
        dfaLearner.setEqKind(eqKind);
        dfaLearner.setBound(bound);
        dfaLearner.setMaxRefined(maxRefined);
        dfaLearner.setDataTime(dataTime);
        //test
        dfaLearner.setWithTest(withTest);
        //saving
        dfaLearner.setSavingDFA(savingDFA);
        //running
        dfaLearner.learning();

    }

    private static void learnerByLength(Model sulModel, String fileName) {
        log.info("Building abstraction ...");
        AlphaBeta<LengthSymbol> mapper;
        switch (symKind) {
            case 0:
                mapper = new RLEMatrixAlphaBeta(sulModel);
                break;
            case 1:
                mapper = new RLESumAlphaBeta(sulModel);
                break;
            case 2:
                mapper = new RLEMeanAlphaBeta(sulModel);
                break;
            default:
                mapper = new RLESumAlphaBeta(sulModel);
                break;
        }
        mapper.setWithFormat(withFormat);
        mapper.setOutSpan(rowSpan, colSpan);
        mapper.setOutSize(rowOut, colOut);
        mapper.setValueScale(valueScale);
        mapper.setDelta(delta);
        mapper.setListLimited(listLimited);

        DFALearner<LengthSymbol> dfaLearner = new DFALearner<>(sulModel, mapper, fileName);
        //alphabet
        dfaLearner.setWithDataAlphabet(withDataAlphabet);
        //algorithm
        dfaLearner.setAlgoKind(algoKind);
        //simulator
        dfaLearner.setpRatio(pRatio);
        dfaLearner.setnRatio(nRatio);
        //membership
        dfaLearner.setWithWord(withWord);
        dfaLearner.setListLimited(listLimited);
        dfaLearner.setWithCache(withCache);
        //equivalence
        dfaLearner.setWithPassiveE(withPassiveE);
        dfaLearner.setTestRatio(testRatio);
        dfaLearner.setEqKind(eqKind);
        dfaLearner.setBound(bound);
        dfaLearner.setMaxRefined(maxRefined);
        dfaLearner.setDataTime(dataTime);
        //test
        dfaLearner.setWithTest(withTest);
        //saving
        dfaLearner.setSavingDFA(savingDFA);
        //running
        dfaLearner.learning();
    }
}
