package org.szu;

import de.learnlib.algorithms.rpni.BlueFringeEDSMDFA;
import de.learnlib.algorithms.rpni.BlueFringeMDLDFA;
import de.learnlib.algorithms.rpni.BlueFringeRPNIDFA;
import net.automatalib.automata.fsa.DFA;
//import de.learnlib.algorithms.rpni.BlueFringeRPNIDFA;
import de.learnlib.api.algorithm.PassiveLearningAlgorithm;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.mnist.dfa.oracle.EvalutionOracle;
import org.szu.mnist.dfa.oracle.MySimulator;
import org.szu.mnist.dfa.symbols.MatrixSymbol;
import org.szu.mnist.mapper.AlphaBeta;
import org.szu.mnist.mapper.SumAlphaBeta;
import org.szu.mnist.sul.DataProvider;
import org.szu.mnist.sul.MnistClassifierModel;
import org.szu.mnist.sul.Model;

import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

public class PassiveLearner {
    private static Logger log = LoggerFactory.getLogger(PassiveLearner.class);
    private int numKind = 1;
    private int orow, ocol;
    private int srow, scol;
    private int vs;
    private int pRatio = 1;
    private int nRatio = 1;
    //algorithm kinds
    private int algoKind = 0;

    public void setNumKind(int numKind) {
        this.numKind = numKind;
    }

    public void setpRatio(int pRatio) {
        this.pRatio = pRatio;
    }

    public void setnRatio(int nRatio) {
        this.nRatio = nRatio;
    }

    public void setAlgoKind(int algoKind) {
        this.algoKind = algoKind;
    }

    public  PassiveLearner(){
        orow = 28;
        ocol = 1;
        srow = 28;
        scol = 1;
        vs = 2;
    }

    public PassiveLearner(int orow, int ocol, int srow, int scol, int vs){
        this.orow = orow;
        this.ocol = ocol;
        this.srow = srow;
        this.scol = scol;
        this.vs = vs;
    }

    public void learning()  {
        DataProvider.setKind(numKind);
        Model sulModel = new MnistClassifierModel();
        AlphaBeta<MatrixSymbol> mapper = new SumAlphaBeta(sulModel);
        mapper.setOutSize(orow, ocol);
        mapper.setOutSpan(srow, scol);
        mapper.setValueScale(vs);

        PassiveLearningAlgorithm.PassiveDFALearner<MatrixSymbol> algorithm = null;
        String algoName = null;
        boolean isPos = false;
        if (algoKind == 2) {
            algorithm = new BlueFringeEDSMDFA<>(mapper.getAlphabetFromData());
            algoName = "_EDSMDFA";
            //this.setpRatio(10);
            //this.setnRatio(1);
        }
        else if (algoKind == 1) {
            algorithm = new BlueFringeMDLDFA<>(mapper.getAlphabetFromData());
            algoName = "_MDLDFA";
            isPos = true;
            //this.setpRatio(10);
            //this.setnRatio(1);
        }
        else {
            algorithm = new BlueFringeRPNIDFA<>(mapper.getAlphabetFromData());
            algoName = "_RPNIDFA";
            //this.setpRatio(10);
            //this.setnRatio(10);
        }

        MySimulator<MatrixSymbol> simulator = new MySimulator<>(mapper, algorithm,isPos); //
        simulator.setpRadio(pRatio);
        simulator.setnRadio(nRatio);
        long start = System.currentTimeMillis();
        DFA<?, MatrixSymbol> pdfa = simulator.getModel();
        long end = System.currentTimeMillis();
        SimpleDateFormat ft = new SimpleDateFormat("-yyMMdd-HHmmss");
        Date date = new Date();
        String sDate = ft.format(date);
        String fileName = "Passive_" + numKind + "_" + orow + "_" + ocol + "_" + srow + "_" + scol + "_" + vs
            + "_" + pRatio + "_" + nRatio + algoName + sDate + ".txt";
        try {
            FileOutputStream fos = new FileOutputStream(fileName);
            PrintWriter out = new PrintWriter(fos);
            log.info("States: " + pdfa.size());
            out.println("States: " + pdfa.size());
            log.info("Times(ms): " + (end-start));
            out.println("Times(ms): " + (end-start));
            EvalutionOracle<MatrixSymbol> tester = new EvalutionOracle<>(out);
            out.println("Testing the passive dfa with training data ...");
            log.info("Testing the passive dfa with training data ...");
            log.info("Counted by words ...");
            out.println("Counted by words ...");
            tester.eval(pdfa, mapper, true);
            log.info("Counted by data ...");
            out.println("Counted by data ...");
            tester.evalByData(pdfa, mapper, true);
            log.info("Testing the passive dfa with testing data ...");
            out.println("Testing the passive dfa with testing data ...");
            log.info("Counted by words ...");
            out.println("Counted by words ...");
            tester.eval(pdfa, mapper, false);
            log.info("Counted by data ...");
            out.println("Counted by data ...");
            tester.evalByData(pdfa, mapper, false);
            out.close();
            fos.close();

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public static void main(String[] args) {
        PassiveLearner learner = new PassiveLearner();
        learner.learning();
    }
}
