package org.szu;

import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.dataset.DataSet;
import org.nd4j.linalg.dataset.api.iterator.DataSetIterator;
import org.nd4j.linalg.factory.Nd4j;
import org.szu.mnist.sul.DataProvider;
import org.szu.mnist.sul.Models4MNIST;
import org.szu.mnist.sul.MnistClassifierModel;
import org.szu.mnist.sul.Model;
import java.util.Random;


public class SafetyAnalyzer {
    private static int k = 2;
    private static int valueScale = 2;
    private static int size = DataProvider.numRows  * DataProvider.numColumns;
    private static int rowOut = 28;
    private static int colOut = 1;
    private static int rowSpan = 28;
    private static int colSpan = 1;
    private static int wordLength = 28;

    private static Random random = new Random();

    public static void valueSafety() throws Exception{
        System.out.println("Hello, DFA for mnist！");
        for(int num = 0; num < 10; num++) {
            System.out.println("Analyzing dataset via num " + num);
            DataProvider.setKind(num);
            System.out.println("Loading system under learning ...");
            Model model = new Models4MNIST(); //new MnistClassifierModel();

            DataSetIterator data = DataProvider.getMnistTrainByKind();
            int batches = DataProvider.getTrainBatches();
            int[] vss = {2, 3, 5, 10};
            int[] kss = {1,10,100,500};

            for (int vi = 0; vi < 4; vi++) {
                valueScale = vss[vi];
                for (int ki = 0; ki < 4; ki++) {
                    k = kss[ki];

                    System.out.println("k: " + k + " valueScale: " + valueScale);

                    float distance = 1.0f / valueScale;
                    int t, flaps, flaps0, total, total0;
                    System.out.println("Distance: " + distance);
                    try {
                        data.reset();
                        t = 0;
                        flaps = 0;
                        flaps0 = 0;
                        total = 0;
                        total0 = 0;
                        while (t < batches) { //data.hasNext()
                            DataSet d = data.next();
                            INDArray inputs = d.getFeatures();
                            INDArray labels = d.getLabels();
                            int length = inputs.rows();
                            for (int i = 0; i < length; i++) {
                                INDArray input = inputs.getRow(i);
                                boolean realRes = Nd4j.argMax(labels.getRow(i), 1).getInt(0) > 0;
                                boolean res4sul = model.query(input);
                                float[] featureData = input.toFloatVector();
                                for (int j = 0; j < k; j++) {
                                    int index = (int) (Math.random() * size);
                                    if (index < 0) index = 0;
                                    if (index >= size) index = size - 1;
                                    float f = featureData[index];
                                    int n = (int) (f / distance);
                                    //System.out.println("f: "+ f + " n: " + n);
                                    float f1 = (float) ((Math.random() + n) * distance);
                                    //System.out.println("f1: "+ f1 + " realDistance: " + (f1 - f));
                                    featureData[index] = f1;
                                }
                                boolean newRes = model.query(Nd4j.create(featureData));
                                total++;
                                if (newRes != res4sul) flaps++;
                                if (realRes == res4sul) {
                                    total0++;
                                    if (newRes != res4sul) flaps0++;
                                }
                            }
                            t++;
                        }
                        data.reset();
                        System.out.println("Total: " + total + " Flaps: " + flaps);
                        System.out.println("Ratio: " + (flaps * 1.0f) / total);
                        System.out.println("Total0: " + total0 + " Flaps0: " + flaps0);
                        System.out.println("Ratio: " + (flaps0 * 1.0f) / total0);
                    } catch (java.lang.Exception e) {
                        e.printStackTrace();
                    }


                }
            }
        }
    }

    private static void randomizeInPlace(float[] data) {
        int len = data.length;
        for (int i = 0; i < len; i++) {
            int pos = random.nextInt(len - i);
            float temp = data[pos];
            data[pos] = data[len-1-i];
            data[len-1-i] = temp;
        }
    }

    public static void blockSafety() throws Exception{
        System.out.println("Hello, DFA for mnist！");
        //for(int num = 0; num < 10; num++) {
            System.out.println("Analyzing dataset via num " + 1);
            DataProvider.setKind(1);
            System.out.println("Loading system under learning ...");
            Model model = new Models4MNIST(); //new MnistClassifierModel();

            DataSetIterator data = DataProvider.getMnistTrainByKind();
            int batches = DataProvider.getTrainBatches();

        int[] paras_row = {28, 28, 14, 28,  7, 28,  4, 28,  2, 28,  1};
        int[] paras_col = {28, 14, 28,  7, 28,  4, 28,  2, 28,  1, 28};
        int len = 11;
        int k;

        for(int pi = 0; pi < len; pi++) {
            rowOut = paras_row[pi]; rowSpan = rowOut;
            colOut = paras_col[pi]; colSpan = colOut;
            wordLength = (size / rowOut) / colOut;

            int pos[] = new int[rowOut * colOut];
            for(int posi = 0; posi < pos.length; posi++){
                int posrow = (posi / rowOut) * DataProvider.numRows;
                int poscol = posi % rowOut;
                pos[posi] = posrow + poscol;
            }
            System.out.print("Position:  ");
            for(int posi = 0; posi < pos.length-1; posi++){
                System.out.print(pos[posi] + " ");
            }
            System.out.println(pos[pos.length-1]);

            for(int kj = 0; kj < wordLength; kj++) {
                k = kj + 1;
                System.out.println("row: " + rowOut + " col: " + colOut + " k:" + k);

                int t, flaps, flaps0, total, total0;
                try {
                    data.reset();
                    t = 0;
                    flaps = 0;
                    flaps0 = 0;
                    total = 0;
                    total0 = 0;
                    while (t < batches) { //data.hasNext()
                        DataSet d = data.next();
                        INDArray inputs = d.getFeatures();
                        INDArray labels = d.getLabels();
                        int length = inputs.rows();
                        for (int i = 0; i < length; i++) {
                            INDArray input = inputs.getRow(i);
                            boolean realRes = Nd4j.argMax(labels.getRow(i), 1).getInt(0) > 0;
                            boolean res4sul = model.query(input);
                            float[] featureData = input.toFloatVector();
                            float tempdata[] = new float[rowOut * colOut];
                            int rownum = DataProvider.numColumns / rowOut;
                            k = kj + 1;
                            for (int j = 0; j < wordLength && k > 0; j++) {
                               if(j + k >= wordLength || random.nextBoolean()) {
                                   int startpos = (j / rownum) * (DataProvider.numRows * colOut) + (j % rownum) * rowOut;
                                  // System.out.println(j + " " + startpos);

                                   for(int fi = 0; fi < tempdata.length; fi++){
                                       tempdata[fi] = featureData[startpos+pos[fi]];
                                   }
                                   /*System.out.print("Before: ");
                                   for(int fi = 0; fi < tempdata.length-1; fi++) {
                                       System.out.print(tempdata[fi] + " ");
                                   }
                                   System.out.println(tempdata[tempdata.length-1]);*/

                                   randomizeInPlace(tempdata);

                                   /*System.out.print("After:  ");
                                   for(int fi = 0; fi < tempdata.length-1; fi++) {
                                       System.out.print(tempdata[fi] + " ");
                                   }
                                   System.out.println(tempdata[tempdata.length-1]);*/

                                   for(int fi = 0; fi < tempdata.length; fi++){
                                       featureData[startpos+pos[fi]] = tempdata[fi];
                                   }
                                   k--;
                               }
                            }
                            boolean newRes = model.query(Nd4j.create(featureData));
                            total++;
                            if (newRes != res4sul) flaps++;
                            if (realRes == res4sul) {
                                total0++;
                                if (newRes != res4sul) flaps0++;
                            }
                        }
                        t++;
                    }
                    data.reset();
                    System.out.println("Total: " + total + " Flaps: " + flaps);
                    System.out.println("Ratio: " + (flaps * 1.0f) / total);
                    System.out.println("Total0: " + total0 + " Flaps0: " + flaps0);
                    System.out.println("Ratio: " + (flaps0 * 1.0f) / total0);
                } catch (java.lang.Exception e) {
                    e.printStackTrace();
                }

            }
        }
        //}
    }
    public static void main(String[] args) throws Exception {
        //valueSafety();
        blockSafety();
    }
}
