package org.szu.mnist.dfa.oracle;

import de.learnlib.api.oracle.EquivalenceOracle;
import de.learnlib.api.query.DefaultQuery;
import net.automatalib.automata.fsa.DFA;
import net.automatalib.words.Word;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.mnist.mapper.AlphaBeta;
import org.szu.mnist.sul.Model;

import javax.annotation.Nullable;
import java.util.*;

//UniversalDeterministicAutomaton<?, I, ?, ?, ?>
public class EquivalenceQueryOracle<I> implements EquivalenceOracle<DFA<?, I>, I, Boolean>  {


    private static Logger log = LoggerFactory.getLogger(EquivalenceQueryOracle.class);

    private Model sulModel = null;
    private AlphaBeta<I> mapper = null;
    private List<Word<I>> dataSet = null;
    private Map<Word<I>, Boolean> dataLables = null;
    private int testingRatio = 4;
    private List<Word<I>> counterExamples;
    //private Random random = new Random();


    public EquivalenceQueryOracle(Model sulModel, AlphaBeta<I> mapper,int testingRatio) {
        try {
            this.sulModel = sulModel;
            this.mapper = mapper;
            this.testingRatio = testingRatio;
            dataLables = mapper.getWordTrainLabels();
            dataSet = new ArrayList<>();
            for(Word word : dataLables.keySet()) {
               if(dataLables.get(word)) dataSet.add(word) ;
            }
            //this.dataset = mapper.getWordTrainSources().values().;
            counterExamples = new ArrayList<>();
        } catch (java.lang.Exception e) {
            e.printStackTrace();
        }
    }

    public void setSulModel(Model sulModel) {
        this.sulModel = sulModel;
    }

    public void setMapper(AlphaBeta<I> mapper) {
        this.mapper = mapper;
    }

    public void setTestingRatio(int testingRatio) {
        this.testingRatio = testingRatio;
    }

    @Nullable
    @Override
    public DefaultQuery<I, Boolean> findCounterExample(DFA<?, I> hypothesis, Collection<? extends I> alphabet) {
        try {
            DefaultQuery<I, Boolean> cexample;
            Word<I> word;
            int caseNum, caseNo, testingNum;
            boolean res4sul, res4hyp;

            //Word<I>[] words = this.dataSet.toArray(new Word[this.dataSet.size()]);
            caseNum = this.dataSet.size(); //.length;
            testingNum = caseNum / this.testingRatio;

            //log.info("Case Number: " + words.length + " Testing Number: " + testingNum);
            int passed = 0;
            for (int i = 0; i < testingNum; i++) {
                caseNo = (int) (Math.random() * caseNum);
                word = this.dataSet.get(caseNo);
                if (!this.dataLables.containsKey(word)) {
                    log.info("No Found Word in Labels: " + word.toString());
                    continue;
                }
                res4hyp = hypothesis.accepts(word);
                res4sul = this.dataLables.get(word);
                //log.info(i + " : "+ caseNo +" : " + word.toString());
               // log.info(String.valueOf(res4sul));
               // log.info(String.valueOf(res4hyp));
                /*realres = Nd4j.argMax(label, 1).getInt(0) > 0;
                res4sul = this.sulModel.query(input);
                word = this.mapper.toWord(input);
                res4hyp = hypothesis.accepts(word);*/
                if(res4hyp == res4sul) passed++;
                if(res4hyp != res4sul && word.length() > 0) {
                    if (!counterExamples.contains(word)) {
                        counterExamples.add(word);
                        log.info(word.toString() + ": " + String.valueOf(res4sul));
                        cexample = new DefaultQuery<>(word, res4sul);
                        log.info("Passed: " + passed);
                        return cexample;
                    }
                }
            }
            log.info("No counter examples. Passed: " + passed);
        } catch (java.lang.Exception e) {
            e.printStackTrace();
            log.info("Error in finding an counter examples");
            return null;
        }
        return null;
    }
}
