package org.szu.mnist.dfa.oracle;

import net.automatalib.automata.fsa.DFA;
import net.automatalib.words.Alphabet;
import net.automatalib.words.Word;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.dataset.api.DataSet;
import org.nd4j.linalg.dataset.api.iterator.DataSetIterator;
import org.nd4j.linalg.factory.Nd4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.Learner;
import org.szu.mnist.dfa.symbols.LengthSymbol;
import org.szu.mnist.mapper.AlphaBeta;
import org.szu.mnist.sul.DataClass;
import org.szu.mnist.sul.DataProvider;
import org.szu.mnist.sul.MyDataFetcher;

import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class EvalutionOracle<I>  {
    private static Logger log = LoggerFactory.getLogger(EvalutionOracle.class);
    PrintWriter out;
    private static DecimalFormat df = new DecimalFormat("0.000");
    //private DataSetIterator testSet;

    public EvalutionOracle(){
        this.out = null;
    }

    public EvalutionOracle(PrintWriter out) {
        this.out = out;
    }

    public void eval(DFA<?, I> model, AlphaBeta<I> mapper, boolean train) {
        int tp, fp,tn,fn;
        boolean res4model,res4real;
        try {
            //Iterator<Word<I>> testSet = mapper.getWordTestSources().keySet().iterator();
            Map<Word<I>,Boolean> testLabels;
            if(train) testLabels = mapper.getWordTrainLabels();
            else testLabels = mapper.getWordTestLabels();
            tp = 0; fp = 0; tn = 0; fn = 0;
            for (Word<I> word : testLabels.keySet()) {
                //if(!testLabels.containsKey(word)) continue;
                res4real = testLabels.get(word);
                res4model = model.accepts(word);
                if(res4real) {
                    if(res4model) tp++;
                    else fn++;
                } else {
                    if(res4model) fp++;
                    else tn++;
                }
            }
            print(tp,tn,fp,fn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void evalByData(DFA<?, I> model, AlphaBeta<I> mapper, boolean train) {
        int tp, fp,tn,fn;
        boolean res4model,res4real;
        try {
            Map<Word<I>,List<DataClass>> testData;
            if(train) testData = mapper.getWordTrainSources();
            else testData = mapper.getWordTestSources();
           // Iterator<Word<I>> testSet = testData.keySet().iterator();
            tp = 0; fp = 0; tn = 0; fn = 0;
            for (Word<I> word: testData.keySet()) {
                //word = testSet.next();
                //if(!testData.containsKey(word)) continue;
                List<DataClass> list = testData.get(word);
                res4model = model.accepts(word);
                for(int i = 0; i<list.size(); i++) {
                    //if(train && !list.get(i).selected) continue;
                    res4real = list.get(i).label;
                    if (res4real) {
                        if (res4model) tp++;
                        else fn++;
                    } else {
                        if (res4model) fp++;
                        else tn++;
                    }
                }
            }
            print(tp,tn,fp,fn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void evals(List<DFA<?, I>> models,  AlphaBeta<I> mapper, boolean train) {
        int tp, fp,tn,fn;
        int tNum, fNum, size;
        boolean res4model,res4real;
        try {
            //Iterator<Word<I>> testSet = mapper.getWordTestSources().keySet().iterator();
            Map<Word<I>,Boolean> testLabels;
            if(train) testLabels = mapper.getWordTrainLabels();
            else testLabels = mapper.getWordTestLabels();
            tp = 0; fp = 0; tn = 0; fn = 0;
            for (Word<I> word : testLabels.keySet()) {
                //if(!testLabels.containsKey(word)) continue;
                res4real = testLabels.get(word);
                tNum = 0; fNum = 0; size = models.size();
                for(int i = 0; i < size; i++) {
                    if(models.get(i).accepts(word)) tNum++;
                    else fNum++;
                }
                res4model = (tNum > fNum); //model.accepts(word);
                if(res4real) {
                    if(res4model) tp++;
                    else fn++;
                } else {
                    if(res4model) fp++;
                    else tn++;
                }
            }
            print(tp,tn,fp,fn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void evalsByData(List<DFA<?, I>> models, AlphaBeta<I> mapper, boolean train) {
        int tp, fp,tn,fn;
        int tNum, fNum, size;
        boolean res4model,res4real;
        try {
            Map<Word<I>,List<DataClass>> testData;
            if(train) testData = mapper.getWordTrainSources();
            else testData = mapper.getWordTestSources();
            // Iterator<Word<I>> testSet = testData.keySet().iterator();
            tp = 0; fp = 0; tn = 0; fn = 0;
            for (Word<I> word: testData.keySet()) {
                //word = testSet.next();
                //if(!testData.containsKey(word)) continue;
                List<DataClass> list = testData.get(word);
                tNum = 0;
                fNum = 0;
                size = models.size();
                for (int i = 0; i < size; i++) {
                    if (models.get(i).accepts(word)) tNum++;
                    else fNum++;
                }
                res4model = (tNum > fNum);
                //res4model = model.accepts(word);
                for (int i = 0; i < list.size(); i++) {
                    //if(train && !list.get(i).selected) continue;
                    res4real = list.get(i).label;
                    if (res4real) {
                        if (res4model) tp++;
                        else fn++;
                    } else {
                        if (res4model) fp++;
                        else tn++;
                    }
                }
            }
            print(tp,tn,fp,fn);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void print(int tp, int tn, int fp,int fn) {
        int total = tp+fp+tn+fn;
        double pre,recall,auc,f1;

        if (tp+fp > 0) pre = (tp * 1.0) /(tp+fp);
        else pre = 1.0;
        if (tp+fn > 0) recall = (tp * 1.0) /(tp+fn);
        else recall = 1.0;
        auc = ((tp+tn)*1.0)/total;
        f1 = 2.0*pre*recall/(pre+recall);


        log.info("TO：" + total);
        log.info("TP：" + tp);
        log.info("FP：" + fp);
        log.info("TN：" + tn);
        log.info("FN：" + fn);

        log.info("pre：" + df.format(pre));
        log.info("recall：" + df.format(recall));
        log.info("auc：" + df.format(auc));
        log.info("F1：" + df.format(f1));

        if(this.out != null) {
            out.println("TP\tTN\tFP\tFN\tpre\trecall\tauc\tf1");
            out.print(tp);
            out.print("\t");
            out.print(tn);
            out.print("\t");
            out.print(fp);
            out.print("\t");
            out.print(fn);
            out.print("\t");
            out.print(df.format(pre));
            out.print("\t");
            out.print(df.format(recall));
            out.print("\t");
            out.print(df.format(auc));
            out.print("\t");
            out.println(df.format(f1));
        }


    }
}
