package org.szu.mnist.dfa.oracle;

import de.learnlib.api.oracle.MembershipOracle;
import de.learnlib.api.query.Query;
import net.automatalib.words.Word;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.mnist.dfa.symbols.LengthSymbol;
import org.szu.mnist.dfa.symbols.MatrixSymbol;
import org.szu.mnist.dfa.symbols.PositionSymbol;
import org.szu.mnist.mapper.AlphaBeta;
import org.szu.mnist.sul.Model;

import java.util.Collection;
import java.util.List;
import java.util.Map;

// use DFAHashCacheOracle<I> for cache.
public class MembershipQueryOracle<I> implements  MembershipOracle<I,Boolean>{
    private static Logger log = LoggerFactory.getLogger(MembershipQueryOracle.class);
    private Model sulModel = null;
    //private INDArray kind = null;
    private AlphaBeta<I> mapper = null;
    private int testcase = 5;
    private boolean withWord = false;
    //private int MaxLength = 784;
    private Map<Word<I>, Boolean> dataLables = null;

    @Override
    public void processQueries(Collection<? extends Query<I, Boolean>> collection) {
        //return this.sulModel.query(this.mapper.toMatrix(input),this.kind);
        //log.info("Processing queries ......");
        for (Query<I, Boolean> query : collection) {
            Word<I> word = query.getInput();
            log.info("Processing the query " + word.toString());
            boolean answer = false;//verify(query.getInput()); //
           if(withWord) {
               answer = verify(query.getInput());
           } else {
               int truenum = 0, falsenum = 0;
                List<INDArray> inputs = this.mapper.toINDArrayList(word);
                if(inputs == null || inputs.size() == 0) {
                    query.answer(answer);
                    continue;
                }
                //log.info("The total possible " + inputs.size());
                int num = inputs.size(); //Math.min(testcase,inputs.size());
                for(int i = 0; i < num; i++) {
                    if (this.sulModel.query(inputs.get(i))) truenum++;
                    else falsenum++;
                }
                answer = (truenum >= falsenum);
            }
            query.answer(answer);
            //log.info("Return the answer " + answer);
        }
    }

    public MembershipQueryOracle(Model sulModel, AlphaBeta<I> mapper) {
        this.sulModel = sulModel;
        this.mapper = mapper;
        dataLables = mapper.getWordTrainLabels();
        this.withWord = false;
    }

    public MembershipQueryOracle(Model sulModel, AlphaBeta<I> mapper, boolean withEnum) {
        this.sulModel = sulModel;
        this.mapper = mapper;
        this.withWord = withEnum;
        dataLables = mapper.getWordTrainLabels();
        /*dataSet = new ArrayList<>();
        for(Word word : dataLables.keySet()) {
            if(dataLables.get(word)) dataSet.add(word) ;
        }*/
    }

    public void setSulModel(Model sulModel) {
        this.sulModel = sulModel;
    }

    public void setWithWord(boolean withWord) {
        this.withWord = withWord;
    }

    public void setMapper(AlphaBeta mapper) {
        this.mapper = mapper;
    }

    public void setTestcase(int testcase) {
        this.testcase = testcase;
    }

    //
    public boolean verify(Word<I> word) {
        if(this.dataLables.containsKey(word)) return this.dataLables.get(word);
        /*for(Word<I> w : this.dataLables.keySet()) {
         if(prefix_zero(word,w)) {
             //log.info(word.toString() + " is substring of " + w.toString());
             return this.dataLables.get(w);
         }
        }*/
        return false;
    }

    //w1 is a prefix of w2, and the remaining of w2 is zero.
    private boolean prefix_zero(Word<I> w1 , Word<I> w2){
        if(w1.length() > w2.length()) return false;
        if(w1.isPrefixOf(w2)) {
            //if(w2 instanceof Word<Integer>) {
                boolean zero = true;
                for (int i = w1.length(); i < w2.length() && zero; i++) {
                    //w1.getSymbol(i).equals()
                    //Integer integer;
                    I symbol = w2.getSymbol(i);
                    if (symbol instanceof MatrixSymbol) {
                        MatrixSymbol value = (MatrixSymbol) symbol;
                        zero = (value.iszero());
                    } else if (symbol instanceof PositionSymbol) {
                        PositionSymbol value = (PositionSymbol) symbol;
                        zero =  value.iszero(); //(value.getValue() == 0);
                    } else if (symbol instanceof LengthSymbol) {
                        LengthSymbol value = (LengthSymbol) symbol;
                        zero = value.iszero(); //(value.getValue() == 0);
                    } else {
                        zero = false;
                    }
                }
                return zero;
            //}
        }
        return false;
    }
}
