package org.szu.mnist.dfa.oracle;

import de.learnlib.api.oracle.EquivalenceOracle;
import de.learnlib.api.oracle.MembershipOracle;
import de.learnlib.api.query.DefaultQuery;
import de.learnlib.api.query.Query;
import de.learnlib.oracle.equivalence.*;
import de.learnlib.oracle.membership.SimulatorOracle;
import net.automatalib.automata.fsa.DFA;
import net.automatalib.util.automata.equivalence.NearLinearEquivalenceTest;
import net.automatalib.words.Word;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.mnist.mapper.AlphaBeta;

import javax.annotation.Nullable;
import java.util.Collection;
import java.util.Random;

public class MyDFAOracle<I> implements MembershipOracle<I,Boolean>, EquivalenceOracle<DFA<?, I>, I, Boolean> {
    private static Logger log = LoggerFactory.getLogger(MyDFAOracle.class);
    private AlphaBeta mapper;
    private DFA<?,I> model;

    private int eqKind = 0;
    private int bound = 1000;

    public MyDFAOracle(AlphaBeta mapper, DFA<?,I> model) {
        this.mapper = mapper;
        this.model = model;
    }

    public void setEqKind(int eqKind) {
        this.eqKind = eqKind;
    }

    public void setBound(int bound) {
        this.bound = bound;
    }

    @Override
    public void processQueries(Collection<? extends Query<I, Boolean>> collection) {
        //return this.sulModel.query(this.mapper.toMatrix(input),this.kind);
        for (Query<I, Boolean> query : collection) {
            log.info("Processing the query " + query.getInput().toString());
            query.answer(this.model.accepts(query.getInput()));
        }
    }

    @Nullable
    @Override
    public DefaultQuery<I, Boolean> findCounterExample(DFA<?, I> dfa, Collection<? extends I> collection) {
        // create a regular membership oracle
        Random random = new Random(123);
        int length = this.mapper.getLength();
        MembershipOracle.DFAMembershipOracle<I> mqOracle = new SimulatorOracle.DFASimulatorOracle<>(this.model);
        // create an equivalence oracle
        DFAEquivalenceOracle<I> eqOracle = null;
        int minLen = 1;
        int rndLen = length + 1 - minLen;
        switch (eqKind) {
            case 0:
                //length / 2, length - (length / 4), (rand.nextDouble() > 1 / (rndLength + 1.0))
                if(length - 2 > minLen) minLen = length - 2;
                if(length + 2 - minLen < rndLen) rndLen = length + 2 - minLen;
                eqOracle = new RandomWpMethodEQOracle.DFARandomWpMethodEQOracle(mqOracle, minLen, rndLen, bound);
                break;
            case 1:
                //length / 2, length - (length / 4),
                if(length - 2 > minLen) minLen = length - 2;
                if(length + 2 - minLen < rndLen) rndLen = length + 2 - minLen;
                eqOracle = new RandomWMethodEQOracle.DFARandomWMethodEQOracle(mqOracle, minLen, rndLen, bound);
                break;
            case 2:
                //length / 2, length+1 ,
                if(length - 2 > minLen) minLen = length - 2;
                eqOracle = new RandomWordsEQOracle.DFARandomWordsEQOracle(mqOracle, minLen, length+2, bound, random);
                break;
            case 3:
                eqOracle = new WpMethodEQOracle.DFAWpMethodEQOracle<>(mqOracle, length+2);
                break;
            case 4:
                eqOracle = new WMethodEQOracle.DFAWMethodEQOracle<>(mqOracle, length+2);
                break;
            case 5:
                //length / 2, length
                if(length - 2 > minLen) minLen = length - 2;
                eqOracle = new CompleteExplorationEQOracle.DFACompleteExplorationEQOracle<>(mqOracle, minLen, length+1);
                break;
            case 6:
                eqOracle = new SimulatorEQOracle.DFASimulatorEQOracle(this.model);
                break;
            default:
                //length / 2, length+1 - (length / 4),
                if(length - 2 > minLen) minLen = length - 2;
                if(length + 2 - minLen < rndLen) rndLen = length + 2 - minLen;
                eqOracle = new RandomWpMethodEQOracle.DFARandomWpMethodEQOracle(mqOracle, minLen, rndLen, bound);
        }
        if (eqKind != 7) {
            return eqOracle.findCounterExample(dfa, collection);
        }
        else {
            System.out.println("==========================NearLinearEquivalenceTest============================");
            NearLinearEquivalenceTest<I> NLET = new NearLinearEquivalenceTest(this.model);
            Word<I> word = NLET.findSeparatingWord(dfa, collection);
            if (word == null) return null;
            return new DefaultQuery<>(word, this.model.accepts(word));
        }
    }
}
