package org.szu.mnist.dfa.oracle;

import de.learnlib.algorithms.rpni.BlueFringeRPNIDFA;
import de.learnlib.api.algorithm.PassiveLearningAlgorithm;
import de.learnlib.api.query.DefaultQuery;
import net.automatalib.automata.fsa.DFA;
import net.automatalib.words.Word;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.mnist.mapper.AlphaBeta;

import java.util.*;

public class MySimulator<I>  {
    private static Logger log = LoggerFactory.getLogger(MySimulator.class);
    private AlphaBeta<I> mapper;
    private PassiveLearningAlgorithm.PassiveDFALearner<I> passiveDFALearner;
    private DFA<?,I> model = null;
    private Random random = new Random();
    private int pRadio;
    private int nRadio;
    private boolean onlyPos = false;

    //todo: different passive learning
    public MySimulator(AlphaBeta<I> mapper, PassiveLearningAlgorithm.PassiveDFALearner<I> passiveDFALearner, boolean isPos) {
        this.mapper = mapper;
        this.passiveDFALearner = passiveDFALearner;
        this.onlyPos = isPos;
        this.pRadio = 1;
        this.nRadio = 1;
    }

    public MySimulator(AlphaBeta<I> mapper) {
        this.mapper = mapper;
        this.passiveDFALearner = new BlueFringeRPNIDFA<>(mapper.getAlphabetFromData());
        //new BlueFringeEDSMDFA<I>(mapper.getAlphabetFromData());
        // BlueFringeMDLDFA (only positive data)
        this.onlyPos = false;
        this.pRadio = 1;
        this.nRadio = 1;
    }

    private void computeModel() {
        Map<Word<I>, Boolean> dataLables = mapper.getWordTrainLabels();
        List<Word<I>> positiveSet = new ArrayList<>();
        List<Word<I>> negativeSet = new ArrayList<>();
        for(Word<I> word : dataLables.keySet()) {
            if(dataLables.get(word)) positiveSet.add(word);
            else negativeSet.add(word);
        }
        //log.info("Positive: " + positiveSet.size());
        //log.info("Negative: " + negativeSet.size());

        //this.passiveDFALearner.addPositiveSamples(positiveSet.subList(0, positiveSet.size()/this.pRadio)); //
        if(this.pRadio >= 10) {
            this.passiveDFALearner.addPositiveSamples(positiveSet);
        }
        else if(this.pRadio > 0 ){
            int pNum = (positiveSet.size() * this.pRadio)/10;
            List<Integer>  selected = select(positiveSet.size(), pNum);
            for(int i = 0; i < selected.size(); i++) {
                this.passiveDFALearner.addPositiveSample(positiveSet.get(selected.get(i).intValue()));
            }
        }
        if(!this.onlyPos) {
            //this.passiveDFALearner.addNegativeSamples(negativeSet.subList(0, negativeSet.size() / this.nRadio));//
            if(this.nRadio >= 10) {
                this.passiveDFALearner.addNegativeSamples(negativeSet);
            }
            else if(this.nRadio > 0 ){
                int nNum = (negativeSet.size() * this.nRadio)/10;
                List<Integer>  selected = select(negativeSet.size(), nNum);
                for(int i = 0; i < selected.size(); i++) {
                    this.passiveDFALearner.addNegativeSample(negativeSet.get(selected.get(i).intValue()));
                }
            }
            this.passiveDFALearner.addNegativeSample(Word.epsilon());//
        }

        //log.info("Computing the model ......");
        this.model = this.passiveDFALearner.computeModel();
       // DFAWriter.printDFA(this.model,this.mapper.getAlphabetFromData(), "MySimulator.txt");
    }

    private List<Integer> select (int n, int k) {
        List<Integer> seeds = new ArrayList<> ();
        for (int i = 0; i<n; i++)  seeds.add(i);
        List<Integer> result = new ArrayList<> ();
        int rand;
        while (result.size() <  k) {
            rand = this.random.nextInt(seeds.size());
            result.add(seeds.get(rand));
            seeds.remove(rand);
        }
        return result;
    }

    public DFA<?,I> getModel() {
        if(model == null) computeModel();
        return model;
    }

    public void refine(DefaultQuery<I, Boolean> query) {
        if(query.getOutput()) {
            this.passiveDFALearner.addPositiveSample(query.getInput());
        }
        else {
            this.passiveDFALearner.addNegativeSample(query.getInput());
        }
        this.model = this.passiveDFALearner.computeModel();

    }

    public void setpRadio(int pRadio) {
        this.pRadio = pRadio;
    }

    public void setnRadio(int nRadio) {
        this.nRadio = nRadio;
    }
}
