package org.szu.mnist.dfa.oracle;

import de.learnlib.api.oracle.EquivalenceOracle;
import de.learnlib.api.oracle.MembershipOracle;
import de.learnlib.api.query.DefaultQuery;
import de.learnlib.api.query.Query;
import de.learnlib.oracle.equivalence.*;
import de.learnlib.oracle.membership.SimulatorOracle;
import net.automatalib.automata.fsa.DFA;
import net.automatalib.util.automata.Automata;
import net.automatalib.util.automata.equivalence.NearLinearEquivalenceTest;
import net.automatalib.words.Word;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.mnist.mapper.AlphaBeta;

import javax.annotation.Nullable;
import java.util.*;

public class MySimulatorOracle<I> implements MembershipOracle<I,Boolean>, EquivalenceOracle<DFA<?, I>, I, Boolean> {
    private static Logger log = LoggerFactory.getLogger(MySimulatorOracle.class);
    private AlphaBeta mapper;
    private DFA<?, I> abstractDFA;
    private MySimulator simulator;

    private int mqKind = 0;
    private int eqKind = 0;
    private int bound = 1000;

    private int refimeTime = 0;
    private int mqRefine = 100;
    private int maxRefind = 20000; //todo: add to setting

    private int dataTime = 10;
    private int curDataTime = 0;

    private boolean withWord = false;
    private boolean withPassiveE = true;

    private int testingRatio = 10;
    private Word<I>[] words;
    private List<Word<I>> counterExamples = null;

    public MySimulatorOracle(AlphaBeta mapper,  MySimulator simulator) {
        this.mapper = mapper;
        this.simulator = simulator;
        this.abstractDFA = this.simulator.getModel();
        Map<Word<I>, Boolean> dataLables = this.mapper.getWordTrainLabels();
        this.words = dataLables.keySet().toArray(new Word[dataLables.size()]);
        counterExamples = new ArrayList<>();//new-xzw
    }

    public void setMqKind(int mqKind) {
        this.mqKind = mqKind;
    }

    public void setEqKind(int eqKind) {
        this.eqKind = eqKind;
    }

    public void setMaxRefind(int maxRefind) {
        this.maxRefind = maxRefind;
    }

    public void setDataTime(int dataTime) {
        this.dataTime = dataTime;
    }

    public void setWithWord(boolean withWord) {
        this.withWord = withWord;
    }

    public void setWithPassiveE(boolean withPassiveE) {
        this.withPassiveE = withPassiveE;
    }

    public void setTestingRatio(int testingRatio) {
        this.testingRatio = testingRatio;
    }

    public void setBound(int bound) {
        this.bound = bound;
    }

    public int getRefimeTime() {
        return refimeTime;
    }

    public DFA<?, I> getAbstractDFA() {
        return abstractDFA;
    }

    @Nullable
    @Override
    public DefaultQuery<I, Boolean> findCounterExample(DFA<?, I> objects, Collection<? extends I> collection) {

        if (!withPassiveE)  return findCounterExampleByData(objects);

        if (curDataTime < dataTime) {
            curDataTime++;
           return findCounterExampleByData(objects);
        } else {
            Random random = new Random(123);
            int length = this.mapper.getLength();
            while (true) {
                // create a regular membership oracle
                MembershipOracle.DFAMembershipOracle<I> mqOracle = new SimulatorOracle.DFASimulatorOracle<>(this.abstractDFA);
                // create an equivalence oracle
                DFAEquivalenceOracle<I> eqOracle = null;
                int minLen = 1;
                int rndLen = length + 1 - minLen;
                switch (eqKind) {
                    case 0:
                        //length / 2, length - (length / 4), (rand.nextDouble() > 1 / (rndLength + 1.0))
                        if(length - 2 > minLen) minLen = length - 2;
                        if(length + 2 - minLen < rndLen) rndLen = length + 2 - minLen;
                        eqOracle = new RandomWpMethodEQOracle.DFARandomWpMethodEQOracle(mqOracle, minLen, rndLen, bound);
                        break;
                    case 1:
                        //length / 2, length - (length / 4),
                        if(length - 2 > minLen) minLen = length - 2;
                        if(length + 2 - minLen < rndLen) rndLen = length + 2 - minLen;
                        eqOracle = new RandomWMethodEQOracle.DFARandomWMethodEQOracle(mqOracle, minLen, rndLen, bound);
                        break;
                    case 2:
                        //length / 2, length+1 ,
                        if(length - 2 > minLen) minLen = length - 2;
                        eqOracle = new RandomWordsEQOracle.DFARandomWordsEQOracle(mqOracle, minLen, length+2, bound, random);
                        break;
                    case 3:
                        eqOracle = new WpMethodEQOracle.DFAWpMethodEQOracle<>(mqOracle, length+2);
                        break;
                    case 4:
                        eqOracle = new WMethodEQOracle.DFAWMethodEQOracle<>(mqOracle, length+2);
                        break;
                    case 5:
                        //length / 2, length
                        if(length - 2 > minLen) minLen = length - 2;
                        eqOracle = new CompleteExplorationEQOracle.DFACompleteExplorationEQOracle<>(mqOracle, minLen, length+1);
                        break;
                    case 6:
                        eqOracle = new SimulatorEQOracle.DFASimulatorEQOracle(this.abstractDFA);
                        break;
                    default:
                        //length / 2, length+1 - (length / 4),
                        if(length - 2 > minLen) minLen = length - 2;
                        if(length + 2 - minLen < rndLen) rndLen = length + 2 - minLen;
                        eqOracle = new RandomWpMethodEQOracle.DFARandomWpMethodEQOracle(mqOracle, minLen, rndLen, bound);
                }

                //Word<I> word = Automata.findSeparatingWord(this.abstractDFA,objects,this.mapper.getAlphabetFromData());
                if (eqKind != 7) {
                    DefaultQuery<I, Boolean> counterExample = eqOracle.findCounterExample(objects, collection);
                    if (counterExample == null) return null;

                    if (this.refimeTime >= this.maxRefind) {
                        return counterExample;
                    }

                    if (counterExamples.contains(counterExample.getInput())) {
                        return counterExample;
                    }

                    boolean res4dfa = this.abstractDFA.accepts(counterExample.getInput());
                    boolean res4sul = this.mapper.query(counterExample.getInput(), withWord);
                    if (res4dfa == res4sul) return counterExample;

                    /*log.info("===================Counter Example=================");
                    log.info(counterExample.toString());
                    log.info("res4dfa: " + res4dfa);
                    log.info("res4sul: " + res4sul);*/

                    log.info("Refine (" + this.refimeTime + ") abstraction DFA with " + counterExample.getInput().toString());
                    counterExample.answer(res4sul);

                    DFA<?, I> tmpModel = this.abstractDFA;
                    try {
                        counterExamples.add(counterExample.getInput()); //new-xzw
                        this.simulator.refine(counterExample);
                        this.abstractDFA = this.simulator.getModel();
                        this.refimeTime++;
                        //return counterExample; //continue with the new abstractDFA
                    } catch (Exception e) {
                        log.info("===========================================refine exception===========================");
                        this.abstractDFA = tmpModel;
                        this.refimeTime = this.maxRefind + 1;
                        counterExample.answer(res4dfa);
                        return counterExample;
                    }
                }
                else { //eqKind == 7
                    System.out.println("==========================NearLinearEquivalenceTest============================");
                    NearLinearEquivalenceTest<I> NLET = new NearLinearEquivalenceTest(this.abstractDFA);
                    Word<I> word = NLET.findSeparatingWord(objects, collection);
                    if (word == null) return null;

                    boolean res4dfa = this.abstractDFA.accepts(word);
                    boolean res4sul = this.mapper.query(word, withWord);
                    DefaultQuery<I, Boolean> counterExample = new DefaultQuery<>(word, res4dfa);
                    if (res4dfa == res4sul) return counterExample;

                    if (counterExamples.contains(word)) return counterExample; //new-xzw

                    if (this.refimeTime >= this.maxRefind) {
                        return counterExample;
                    }
                    /*log.info("===================Counter Example=================");
                    log.info(counterExample.toString());
                    log.info("res4dfa: " + res4dfa);
                    log.info("res4sul: " + res4sul);*/
                    log.info("Refine (" + this.refimeTime + ") abstraction DFA with " + counterExample.getInput().toString());
                    counterExample.answer(res4sul);

                    DFA<?, I> tmpModel = this.abstractDFA;
                    try {
                        this.counterExamples.add(word);//new-xzw
                        this.simulator.refine(counterExample);
                        this.abstractDFA = this.simulator.getModel();
                        this.refimeTime++;
                        //return counterExample;  //continue with the new abstractDFA
                    } catch (Exception e) {
                        log.info("===========================================refine exception===========================");
                        this.abstractDFA = tmpModel;
                        this.refimeTime = this.maxRefind + 1;
                        counterExample.answer(res4dfa);
                        return counterExample;
                    }
                }
            }
        }
       // return null;
    }


    private DefaultQuery<I, Boolean> findCounterExampleByData(DFA<?, I> objects) {
        Word<I> word;
        int caseNum, caseNo, testingNum;
        boolean res4sul, res4hyp;
        caseNum = this.words.length;
        testingNum = caseNum / this.testingRatio;
        for (int i = 0; i < testingNum; i++) {
            caseNo = (int) (Math.random() * caseNum);
            word =this.words[caseNo];
            res4hyp = objects.accepts(word);
            res4sul = this.mapper.query(word,withWord); //true
            if (res4hyp != res4sul && word.length() > 0) {
                if (!counterExamples.contains(word)) { //??
                    counterExamples.add(word);
                    log.info("Found Counterexample: " + word.toString() + ": " + String.valueOf(res4sul));
                    return new DefaultQuery<>(word, res4sul);
                }
            }
        }
        return null;
    }

    //for membership
    @Override
    public void processQueries(Collection<? extends Query<I, Boolean>> collection) {
        //log.info("Processing queries ......");
        for (Query<I, Boolean> query : collection) {
            Word<I> word = query.getInput();
            log.info("Processing the query " + word.toString());
            if(withPassiveE) {
                if(this.refimeTime >= this.maxRefind || this.counterExamples.contains(word) ) { //|| this.mqKind > 0
                    query.answer(this.abstractDFA.accepts(word));
                } else {
                    boolean res = this.mapper.query(word,withWord);
                    //// to refine the abstractDFA in case of Incompatible acceptances
                    boolean res4dfa = this.abstractDFA.accepts(word);
                    if(res4dfa != res) {
                        DefaultQuery<I, Boolean> counterExample = new DefaultQuery<>(word);
                        counterExample.answer(res);
                        if(this.mqRefine <= 0) log.info("Refine (" + this.refimeTime + ") abstraction DFA with " + counterExample.getInput().toString());
                        DFA<?,I> tmpModel = this.abstractDFA;
                        try {
                            this.simulator.refine(counterExample);
                            this.abstractDFA = this.simulator.getModel();
                            this.mqRefine--;
                            if(this.mqRefine < 0) this.refimeTime++;
                            query.answer(res);
                        } catch (Exception e) {
                            log.info("===========================================refine exception===========================");
                            this.abstractDFA = tmpModel;
                            this.refimeTime = this.maxRefind+1;
                            query.answer(res4dfa);
                        }
                    } else {
                        query.answer(res);
                    }
                }
            }
            else {
                query.answer(this.mapper.query(word,withWord));
            }
        }
    }

}
