package org.szu.mnist.dfa.symbols;

import net.automatalib.words.abstractimpl.AbstractSymbol;
import org.jetbrains.annotations.NotNull;

import javax.annotation.Nonnull;
import java.math.BigInteger;
import java.util.Objects;

public class LengthSymbol extends AbstractSymbol<LengthSymbol> {
    private int len;
    private int value;
    private boolean withLength = true;
    private BigInteger value_BIG;
    private boolean isBIG = false;

    public LengthSymbol(int value, int len) {
        this.value = value;
        this.len = len;
        this.isBIG = false;
    }

    public LengthSymbol(BigInteger value, int len) {
        this.value_BIG = value;
        this.len = len;
        this.isBIG = true;
    }

    public void setLen(int len) {
        this.len = len;
    }

    public int getLen() {
        return len;
    }

    public boolean isWithLength() {
        return withLength;
    }

    public void setWithLength(boolean withLength) {

        this.withLength = withLength;
    }

    @Override
    public int compareTo(@NotNull LengthSymbol o) {
        //ignore the length
        if(withLength) {
            if(this.len != o.len) return this.len - o.len;
        }
        if(isBIG) return this.value_BIG.compareTo(o.value_BIG);
        return this.value - o.value;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        LengthSymbol that = (LengthSymbol) o;
        if(withLength) {
            if(this.len != that.len)  return false;
        }
        if(isBIG) return this.value_BIG.equals(that.value_BIG);
        return this.value == that.value;
    }

    @Override
    public int hashCode() {
        if(withLength) {
            if(isBIG) return Objects.hash(value_BIG,len);
            return Objects.hash(value,len);
        }
        if(isBIG) return Objects.hash(value_BIG);
        return Objects.hash(value);
    }

    @Override
    @Nonnull
    public String toString() {
        if(withLength) {
            if(isBIG) return "[" + value_BIG.toString() + ":" + len + "]";
            return "[" + value + ":" + len + "]";
        }
        if(isBIG) return "[" + value_BIG.toString() + ":" + len + "]";
        return String.valueOf(value);
    }

    public int getValue() {
        return value;
    }

    public BigInteger getValue_BIG() {
        return value_BIG;
    }

    public boolean iszero() {
        if(isBIG) return this.value_BIG.equals(BigInteger.ZERO);
        return this.value == 0;
    }
}
