package org.szu.mnist.dfa.symbols;

import net.automatalib.words.abstractimpl.AbstractSymbol;
import org.jetbrains.annotations.NotNull;

import java.math.BigInteger;
import java.util.Objects;

public class MatrixSymbol extends AbstractSymbol<MatrixSymbol> {
    private int value;
    private BigInteger value_BIG;
    private boolean isBIG = false;

    public MatrixSymbol(int value) {
        this.value = value;
        this.isBIG = false;
    }

    public MatrixSymbol(BigInteger value_BIG) {
        this.value_BIG = value_BIG;
        this.isBIG = true;
    }

    @Override
    public int compareTo(@NotNull MatrixSymbol o) {
        if(isBIG) return this.value_BIG.compareTo(o.value_BIG);
        return this.value - o.value;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        MatrixSymbol that = (MatrixSymbol) o;
        if(isBIG) return this.value_BIG.equals(that.value_BIG);
        return value == that.value ;
    }

    @Override
    public int hashCode() {
        if(isBIG) return Objects.hash(value_BIG);
        return Objects.hash(value);
    }

    @Override
    public String toString() {
        if(isBIG) return value_BIG.toString();
        return String.valueOf(value);
    }

    public int getValue() {
        return value;
    }

    public BigInteger getValue_BIG() {
        return value_BIG;
    }

    public boolean iszero() {
        if(isBIG) return this.value_BIG.equals(BigInteger.ZERO);
        return this.value == 0;
    }
}
