package org.szu.mnist.mapper;

import net.automatalib.words.Word;
import net.automatalib.words.Alphabet;
import net.automatalib.words.impl.Alphabets;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.dataset.api.DataSet;
import org.nd4j.linalg.dataset.api.iterator.DataSetIterator;
import org.nd4j.linalg.factory.Nd4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.szu.Learner;
import org.szu.mnist.sul.DataClass;
import org.szu.mnist.sul.DataProvider;
import org.szu.mnist.sul.Model;


import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.*;

public abstract class AlphaBeta<I> {
    private static Logger log = LoggerFactory.getLogger(AlphaBeta.class);

    protected final int rowin = DataProvider.numRows;
    protected final int colin = DataProvider.numColumns;

    protected Alphabet<I> alphabet = null;
    protected Alphabet<I> alphabetFromData = null;

    protected int rowout = 4;
    protected int colout = 4;

    protected int rowspan = 4;
    protected int colspan = 4;

    protected int valuescale = 2;


    protected int listLimited = 99;
    protected float delta = 0.25f; //should in [0, 1.0]

    protected int length;

    protected boolean withFormat = true;

    //keep the data source for mapping
    //protected  boolean saveWordSource = false;
    protected Map<Word<I>, List<DataClass>> wordTrainSources = null;
    protected Map<Word<I>, List<DataClass>> wordTestSources = null;
    protected Map<Word<I>, Boolean> wordTrainLabels = null;
    protected Map<Word<I>, Boolean> wordTestLabels = null;
    //protected
    protected boolean saveSource = false;
    protected Map<Integer, List<float[][]>> sources =  new HashMap<>();
    protected Map<BigInteger,List<float[][]>> sources_BIG = new HashMap<>();


    protected Model model;

    public void setDelta(float delta) {
        if(delta >= 0 && delta <= 1) this.delta = delta;
    }
    public void setWithFormat(boolean withFormat) {
        this.withFormat = withFormat;
    }

    public void setOutSize(int rowout, int colout) {
        this.rowout = rowout;
        this.colout = colout;
        length = getLength();
    }

    public void setOutSpan(int rowspan, int colspan) {
        this.rowspan = rowspan;
        this.colspan = colspan;
        length = getLength();
    }

    public boolean isLargeSym() {
        if (this.rowout >= 14 && this.colout >= 14) return true;
        else return false;
    }

    public int getLength() {
        int rownum = (rowin-rowout+1);
        int rown = rownum / rowspan;
        if(rownum % rowspan > 0) rown++;
        int colnum = colin-colout+1;
        int coln = colnum / colspan;
        if(colnum % colspan > 0) coln++;
        int maxLength = rown * coln;
        return maxLength;
    }

    public void setValueScale(int valuescale) {
        this.valuescale = valuescale;
    }


    private void dataAnalysis(DataSetIterator data, int batches, boolean train) {
        log.info("Analyzing the data ......");
        DataSet d;
        INDArray inputs,labels,input;
        int length, t;
        boolean res, res4sul;

        Map<Word<I>, List<DataClass>> wordSources;
        Map<Word<I>, Boolean> wordLables;
        if(train) wordSources = this.wordTrainSources;
        else wordSources = this.wordTestSources;
        if(train) wordLables = this.wordTrainLabels;
        else wordLables = this.wordTestLabels;

        try {
            data.reset();  t = 0;
            while (t < batches) { //data.hasNext()
                d = data.next();
                inputs = d.getFeatures();
                labels = d.getLabels();
                length = inputs.rows();
                for (int i = 0; i < length; i++) {
                    input = inputs.getRow(i);
                    Word<I> word = toWord(input);
                    //log.info(word.toString());
                    res = Nd4j.argMax(labels.getRow(i),1).getInt(0) > 0;
                    if(train) res4sul = model.query(input);
                    else res4sul = false;
                    if(wordSources.containsKey(word)) {
                        List<DataClass> wordList = wordSources.get(word);
                        wordList.add(new DataClass(input,res,res4sul));
                    } else {
                        List<DataClass> wordList = new ArrayList<>();
                        wordList.add(new DataClass(input,res,res4sul));
                        wordSources.put(word, wordList);
                    }
                }
                t++;
            }
            data.reset();
        } catch (java.lang.Exception e) {
            e.printStackTrace();
        }
       // if(train) wordTrainLabels.clear();
        DataClass pic; boolean selected;
        int positive, negative,pNum,nNum,ptotal,ntotal;
        int total, pconflicts,nconflicts,pdconflicts,ndconflicts;
        pNum = 0; nNum = 0; total = 0; pconflicts = 0; nconflicts = 0;
        pdconflicts = 0; ndconflicts = 0; ptotal = 0; ntotal = 0;
        for(Word word : wordSources.keySet()) {
            positive = 0; negative = 0;
            List<DataClass> list = wordSources.get(word);
            int size = list.size();
            for(int i = 0; i < size; i++) {
                pic = list.get(i);
                if(train && pic.label != pic.res4sul) continue;
                total++;
                if(pic.label) {
                    positive++;
                    ptotal++;
                }
                else {
                    negative++;
                    ntotal++;
                }
            }
            if(positive + negative == 0) continue;
            selected = (positive >= negative);
            if(positive > 0 && negative > 0) {
                if(selected) {
                    pconflicts++;
                    ndconflicts += negative;
                }
                else {
                    nconflicts++;
                    pdconflicts += positive;
                }
            }
            for(int i = 0; i < size; i++) {
                pic = list.get(i);
                if(pic.label == selected && (!train || selected == pic.res4sul)) pic.selected = true;
            }
            //if(train) wordTrainLabels.put(word,selected);
            if(selected) pNum++;
            else nNum++;
            wordLables.put(word,selected);
        }
        log.info("Correct Data in total: " + total);
        log.info("Positive Data: " + ptotal + " Negative Data: " + ntotal);
        log.info("Conflict Positive Data: " + pdconflicts + " Conflict Negative Data: " + ndconflicts);
        log.info("Words in total: " + wordSources.keySet().size());
        log.info("Corrected Words in total: " + wordLables.keySet().size());
        log.info("Conflict Positive Words: " + pconflicts + " Conflict Negative Words: " + nconflicts);
        log.info("Positive Words: " + pNum + " Negative Words: " + nNum);

        /*StringBuffer buffer = new StringBuffer();
        SimpleDateFormat ft = new SimpleDateFormat("-yyMMdd-HHmmss");
        Date date = new Date();
        String sDate = ft.format(date);
        buffer.append("DataInfo");
        buffer.append("_");
        buffer.append(this.rowout);
        buffer.append("_");
        buffer.append(this.colout);
        buffer.append("_");
        buffer.append(rowspan);
        buffer.append("_");
        buffer.append(colspan);
        buffer.append("_");
        buffer.append(valuescale);
        buffer.append("_");
        buffer.append(train);
        buffer.append("_");
        buffer.append(DataProvider.getKind());
        buffer.append("_");
        buffer.append(sDate);
        buffer.append(".txt");
        try{
            FileOutputStream fos = new FileOutputStream(buffer.toString());
            PrintWriter out = new PrintWriter(fos);
            out.println("Correct Data in total: " + total);
            out.println("Positive Data: " + ptotal + " Negative Data: " + ntotal);
            out.println("Conflict Positive Data: " + pdconflicts + " Conflict Negative Data: " + ndconflicts);
            out.println("Words in total: " + wordSources.keySet().size());
            out.println("Corrected Words in total: " + wordLables.keySet().size());
            out.println("Conflict Positive Words: " + pconflicts + " Conflict Negative Words: " + nconflicts);
            out.println("Positive Words: " + pNum + " Negative Words: " + nNum);
            out.close();
            fos.close();
        }catch(Exception e){
            e.printStackTrace();
        }*/


    }

    protected  void setWordSources() {
        if(wordTrainSources != null && wordTestSources != null &&
            wordTrainLabels != null && wordTestLabels != null) return;
        try {
            if(wordTrainSources != null) wordTrainSources.clear();
            else wordTrainSources = new HashMap<>();
            if(wordTrainLabels != null) wordTrainLabels.clear();
            else wordTrainLabels = new HashMap<>();
            DataSetIterator data = DataProvider.getMnistTrainByKind();
            dataAnalysis(data,DataProvider.getTrainBatches(),true);
            if(wordTestSources != null) wordTestSources.clear();
            else wordTestSources = new HashMap<>();
            if(wordTestLabels != null) wordTestLabels.clear();
            else wordTestLabels = new HashMap<>();
            data = DataProvider.getMnistTestByKind();
            dataAnalysis(data,DataProvider.getTestBatches(),false);
        }catch (java.lang.Exception e) {
            e.printStackTrace();
        }
    }
    ////
    private Set<I> getSymbolsFromData(Map<Word<I>, List<DataClass>> wordSources) {
        log.info("Analyzing the symbols ......");

        Set<I> symbols = new HashSet<>();
        int length;
        List<DataClass> list;
        DataClass d;

        for(Word<I> word : wordSources.keySet()) {
            list = wordSources.get(word);
            length = list.size();
            for (int i = 0; i < length; i++) {
                d = list.get(i);
                if(d.selected) symbols.addAll(toSymbolList(d.input));
            }
        }

        log.info("Symbols in total: " + symbols.size());
        return symbols;
    }

    ////
    public Alphabet<I> getAlphabetFromData(){
        if(alphabetFromData != null) return alphabetFromData;

        boolean saveSourceBak = this.saveSource;
        this.saveSource = true;

        Set<I> symbols = new HashSet<>();
        if(wordTrainSources == null || wordTestSources == null) setWordSources();
        symbols.addAll(getSymbolsFromData(this.wordTrainSources));
        symbols.addAll(getSymbolsFromData(this.wordTestSources));

        this.saveSource = saveSourceBak;
        alphabetFromData = Alphabets.fromCollection(symbols);
        return  alphabetFromData;
    }

    public Map<Word<I>, List<DataClass>> getWordTrainSources() {
        if(wordTrainSources == null) setWordSources();
        return wordTrainSources;
    }

    public Map<Word<I>, List<DataClass>> getWordTestSources() {
        if(wordTestSources == null) setWordSources();
        return wordTestSources;
    }

    public Map<Word<I>, Boolean> getWordTrainLabels() {
        if(wordTrainLabels == null) setWordSources();
        return wordTrainLabels;
    }

    public Map<Word<I>, Boolean> getWordTestLabels() {
        if(wordTestLabels == null) setWordSources();
        return wordTestLabels;
    }

    protected float distance(float[][] f1, float[][] f2) {
        if(f1 == null || f2 == null) return 0f;
        int row = Math.min(f1.length, f2.length);
        int col = Math.min(f1[0].length, f2[0].length);
        if( row == 0 || col == 0) return 0f;
        float diff = 0f;
        for(int i = 0; i < row; i++) {
            for(int j = 0; j < col; j++){
                diff += Math.abs(f1[i][j] - f2[i][j]);
            }
        }
        return diff / (row * col);
    }

    protected void addList(List<float[][]> list, float[][] f) {
        boolean addable = true;
        int length = list.size();
        for(int i = 0; i < length && addable; i++) {
            if(distance(f,list.get(i)) <= 1.0f / this.valuescale) addable = false;
        }
        if(addable) list.add(f);
    }

    public void setListLimited(int listLimited) {
        this.listLimited = listLimited;
    }

    public abstract BigInteger getAlphabetSize();

    public abstract Alphabet<I> getAlphabet();

    protected abstract I toSymbol(INDArray data,int rowi,int coli);

    public abstract List<I> toSymbolList(INDArray data);

    public Word<I> toWord(INDArray data) {
        return Word.fromList(toSymbolList(data)) ;
    }

    public abstract INDArray toINDArray(Word<I> word);
    public abstract List<INDArray> toINDArrayList(Word<I> word);
    public abstract boolean query(Word<I> word, boolean withWord);


}

