package org.szu.mnist.mapper;

import net.automatalib.words.Alphabet;
import net.automatalib.words.Word;
import net.automatalib.words.impl.Alphabets;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.factory.Nd4j;
import org.szu.mnist.dfa.symbols.MatrixSymbol;
import org.szu.mnist.dfa.symbols.PositionSymbol;
import org.szu.mnist.sul.DataClass;
import org.szu.mnist.sul.Model;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;


public class MatrixAlphaBeta extends AlphaBeta<MatrixSymbol> {
    private BigInteger symbolscale;
    private int length;

    public MatrixAlphaBeta(Model model) {
        rowout = 4;
        colout = 4;
        rowspan = 4;
        colspan = 4;
        valuescale = 2;
        BigInteger valueTemp = BigInteger.valueOf(this.valuescale);
        symbolscale = valueTemp.pow(this.rowout * this.colout);
        //this.symbolscale = (int) Math.pow(this.valuescale, this.rowout * this.colout);
        length = getLength();
        this.model = model;
    }


    @Override
    public void setOutSize(int rowout, int colout) {
        super.setOutSize(rowout,colout);
        BigInteger valueTemp = BigInteger.valueOf(this.valuescale);
        symbolscale = valueTemp.pow(this.rowout * this.colout);
        //this.symbolscale = (int) Math.pow(this.valuescale, this.rowout * this.colout);
    }

    @Override
    public void setValueScale(int valuescale) {
        this.valuescale = valuescale;
        BigInteger valueTemp = BigInteger.valueOf(this.valuescale);
        symbolscale = valueTemp.pow(this.rowout * this.colout);
        //this.symbolscale = (int) Math.pow(this.valuescale, this.rowout * this.colout);
    }


    @Override
    public Alphabet<MatrixSymbol> getAlphabet(){
        if(alphabet != null) return alphabet;
        List<MatrixSymbol> symbols = new ArrayList<>();
        BigInteger temp = this.symbolscale;
        while(true) {
            if(temp.equals(BigInteger.ZERO)) break;
            temp = temp.subtract(BigInteger.ONE);
            symbols.add(new MatrixSymbol(temp));
        }
       // alphabet = Alphabets.integers(0, this.symbolscale-1);
        alphabet = Alphabets.fromList(symbols);
        return alphabet;
    }


    @Override
    protected MatrixSymbol toSymbol(INDArray data, int rowi, int coli){
        BigInteger value, scale;
        int inc;
        int rown, coln;
        float f;
        float[][] source = null;

        rown = Math.min(this.rowout, this.rowin-rowi);
        coln = Math.min(this.colout, this.colin-coli);
        if(this.saveSource) source = new float[rown][coln];
        value = BigInteger.ZERO;
        scale = BigInteger.valueOf(this.valuescale);
        for(int i = 0; i < this.rowout; i++) {
            for(int j = 0; j < this.colout; j++) {
                if(rowi+i >= this.rowin || coli+j >= this.colin) value = value.multiply(scale);
                else {
                    f = data.getFloat((rowi + i) * this.colin + coli + j);
                    if (this.saveSource) source[i][j] = f;
                    inc = (int) (f * this.valuescale);
                    if (inc >= this.valuescale) inc = this.valuescale - 1;
                    value = value.multiply(scale).add(BigInteger.valueOf(inc));
                }
            }
        }
        if(this.saveSource && !value.equals(BigInteger.ZERO)) {
            if(!this.sources_BIG.containsKey(value)) {
                List<float[][]> list = new ArrayList<>();
                list.add(source);
                this.sources_BIG.put(value,list);
            } else {
                List<float[][]>  list = this.sources_BIG.get(value);
                //list.add(source);
                addList(list,source);
            }
        }
        return new MatrixSymbol(value);
    }

    protected  List<List<float[][]>> getData(Word<MatrixSymbol> word) {
        BigInteger value,scale;
        int len, roi, coi;
        boolean done;
        float[][] data;
        List<List<float[][]>> data4word = new ArrayList<>();
        len = word.length();
        for (int i = 0; i < len; i++) {
            value = word.getSymbol(i).getValue_BIG();
            if (this.sources_BIG.containsKey(value)) {
                if(this.sources_BIG.get(value).size() <= 0) return null;
                data4word.add(this.sources_BIG.get(value));
            }
            else {
                data = new float[this.rowout][this.colout];
                roi = this.rowout - 1;
                coi = this.colout - 1;
                done = false;
                scale = BigInteger.valueOf(this.valuescale);
                while (value.compareTo(BigInteger.ZERO) > 0 && !done) {
                    data[roi][coi] = (float) ((value.remainder(scale).floatValue() + Math.random() * delta) / this.valuescale);
                    value = value.divide(scale);
                    coi--;
                    if (coi < 0) {
                        coi = this.colout - 1;
                        roi--;
                        if (roi < 0) done = true;
                    }
                }
                List<float[][]> dataList = new ArrayList<>();
                dataList.add(data);
                data4word.add(dataList);
            }
        }
        return data4word;
    }

    @Override
    public List<INDArray> toINDArrayList(Word<MatrixSymbol> word) {
        if(word.length() == 0) return null;
        if(this.withFormat && word.length() != length) return null;
        if(this.wordTrainSources!=null && this.wordTrainSources.containsKey(word)) {
            List<DataClass> list = this.wordTrainSources.get(word);
            List<INDArray> wordlist = new ArrayList<>();
            for(int i = 0; i <list.size(); i++) if(list.get(i).selected) wordlist.add(list.get(i).input);
            if(wordlist.size() > this.listLimited) return wordlist.subList(0,this.listLimited-1);
            return wordlist;
        }


        List<List<float[][]>> data4word = getData(word);
        if(data4word == null) return null;
        int len = data4word.size();
        if(len <= 0) return null;
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for(int i = 0; i< len; i++) {
            dataIndexes[i] = 0;
            dataLens[i] = data4word.get(i).size();
        }

        List<INDArray> indArrayList = new ArrayList<>();
        float[] featureData;
        int[] featureCounted;
        int index,rowi,coli,rown,coln;
       while(true) {
           featureData = new float[this.rowin * this.colin];
           featureCounted = new int[this.rowin * this.colin];
           for(int i = 0; i < featureData.length; i++) {
               featureData[i] = 0.0f;
               featureCounted[i] = 0;
           }
           index = 0;  rowi = 0;
           while (rowi < this.rowin-this.rowout+1 && index < len) {
               coli = 0;
               while (coli < this.colin-this.colout+1 && index < len) {
                   float[][] source = data4word.get(index).get(dataIndexes[index]);
                   index++;
                   rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                   coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                   for (int roi = 0; roi < rown; roi++) {
                       for (int coi = 0; coi < coln; coi++) {
                           featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                           featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                       }
                   }
                   coli = coli + this.colspan;
               }
               rowi = rowi + this.rowspan;
           }
           for(int i = 0; i < featureData.length; i++) {
               if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
           }
           indArrayList.add(Nd4j.create(featureData));
           boolean done = true;
            for(int i = 0; i<len && done; i++) {
                if(dataIndexes[i] < dataLens[i] -1) done = false;
            }
            if(done || indArrayList.size() >= this.listLimited) break;
            int roi = 0;
           while(true) {
               dataIndexes[roi]++;
               if(dataIndexes[roi] >= dataLens[roi]) {
                   dataIndexes[roi] = 0;
                   roi++; continue;
               }
               break;
           }
       }
       return indArrayList;
    }

    @Override
    public INDArray toINDArray(Word<MatrixSymbol> word) {
            if (word.length() == 0) return null;
            if (this.withFormat && word.length() != length) return null;
            if (this.wordTrainSources != null && this.wordTrainSources.containsKey(word)) {
                List<DataClass> list = this.wordTrainSources.get(word);
                List<DataClass> wordlist = new ArrayList<>();
                for (int i = 0; i < list.size(); i++) if (list.get(i).selected) wordlist.add(list.get(i));
                if (wordlist.size() > 0) {
                    int selected = (int) (Math.random() * wordlist.size());
                    return wordlist.get(selected).input;
                }
            }

            float[] featureData = new float[this.rowin * this.colin];
            int[] featureCounted = new int[this.rowin * this.colin];
            for (int i = 0; i < featureData.length; i++) {
                featureData[i] = 0.0f;
                featureCounted[i] = 0;
            }

            try {
                int rowi, coli, i;
                int index, length;
                BigInteger value, scale;
                int roi, coi;
                boolean done;

                index = 0;
                length = word.length();
                rowi = 0;
                if (length == 0) return Nd4j.create(featureData);
                while (rowi < this.rowin-this.rowout+1 && index < length) {///
                    coli = 0;
                    while (coli < this.colin-this.colout+1 && index < length) {///
                        value = word.getSymbol(index).getValue_BIG();
                        index++;
                        if (this.sources_BIG.containsKey(value)) {
                            List<float[][]> list = this.sources_BIG.get(value);
                            int selected = (int) (Math.random() * list.size());
                            float[][] source = list.get(selected);
                            int rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                            int coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                            for (roi = 0; roi < rown; roi++) {
                                for (coi = 0; coi < coln; coi++) {
                                    // if (rowi + roi < this.rowin && coli + coi < this.colin && roi < source.length && coi < source[0].length) {
                                    featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                                    featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                                    // }
                                }
                            }
                        } else {
                            roi = this.rowout - 1;
                            coi = this.colout - 1;
                            done = false;
                            scale = BigInteger.valueOf(this.valuescale);
                            while (value.compareTo(BigInteger.ZERO) > 0 && !done) {
                                if (rowi + roi < this.rowin && coli + coi < this.colin) { //random.nextDouble() / 2
                                    i = (rowi + roi) * this.colin + (coli + coi);
                                    featureData[i] += (float) ((value.remainder(scale).floatValue() + Math.random() * delta) / this.valuescale);
                                    featureCounted[i]++;
                                }
                                value = value.divide(scale);
                                coi--;
                                if (coi < 0) {
                                    coi = this.colout - 1;
                                    roi--;
                                    if (roi < 0) done = true;
                                }
                            }
                        }
                        coli = coli + this.colspan;
                    }
                    rowi = rowi + this.rowspan;
                }
            } catch (java.lang.Exception e) {
                e.printStackTrace();
            }
            for (int i = 0; i < featureData.length; i++) {
                if (featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
            }
            return Nd4j.create(featureData);
        }

    @Override
    public List<MatrixSymbol> toSymbolList(INDArray data) {
        List<MatrixSymbol> symbols = new ArrayList<>();
        int rowi, coli;

        rowi = 0;
        while(rowi < this.rowin-this.rowout+1) { ///
            coli = 0;
            while (coli < this.colin-this.colout+1) {////
                symbols.add(toSymbol(data,rowi,coli));
                coli += this.colspan;
            }
            rowi += this.rowspan;
        }
        return symbols;
    }

    @Override
    public BigInteger getAlphabetSize(){
        return this.symbolscale;
    }

    @Override
    public boolean query(Word<MatrixSymbol> word, boolean withWord) {
        if(word.length() == 0) return false;

        if(this.withFormat &&  word.length() != length) return false;

        if(wordTrainLabels.containsKey(word)) {
            return wordTrainLabels.get(word);
        }

        if(withWord) return false;

        List<List<float[][]>> data4word = getData(word);
        if(data4word == null) return false;
        int len = data4word.size();
        if(len <= 0) return false;
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for(int i = 0; i< len; i++) {
            dataIndexes[i] = 0;
            dataLens[i] = data4word.get(i).size();
        }

        float[] featureData;
        int[] featureCounted;
        int index, rowi, coli, rown, coln;
        int tNum, fNum;
        tNum = 0; fNum = 0;
        while(true) {
            featureData = new float[this.rowin * this.colin];
            featureCounted = new int[this.rowin * this.colin];
            for (int i = 0; i < featureData.length; i++) {
                featureData[i] = 0.0f;
                featureCounted[i] = 0;
            }
            index = 0;
            rowi = 0;
            while (rowi < this.rowin - this.rowout + 1 && index < len) {
                coli = 0;
                while (coli < this.colin - this.colout + 1 && index < len) {
                    float[][] source = data4word.get(index).get(dataIndexes[index]);
                    index++;
                    rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                    coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                    for (int roi = 0; roi < rown; roi++) {
                        for (int coi = 0; coi < coln; coi++) {
                            featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                            featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                        }
                    }
                    coli = coli + this.colspan;
                }
                rowi = rowi + this.rowspan;
            }
            for (int i = 0; i < featureData.length; i++) {
                if (featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
            }
            if(this.model.query(Nd4j.create(featureData))) tNum++;
            else fNum++;
            boolean done = true;
            for (int i = 0; i < len && done; i++) {
                if (dataIndexes[i] < dataLens[i] - 1) done = false;
            }
            if (done || (tNum + fNum) >= this.listLimited) break;
            int di = 0;
            while (true) {
                dataIndexes[di]++;
                if (dataIndexes[di] >= dataLens[di]) {
                    dataIndexes[di] = 0;
                    di++;
                    continue;
                }
                break;
            }
        }
        return tNum > fNum;
    }
}
