package org.szu.mnist.mapper;

import net.automatalib.words.Alphabet;
import net.automatalib.words.Word;
import net.automatalib.words.impl.Alphabets;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.factory.Nd4j;
import org.szu.mnist.dfa.symbols.MatrixSymbol;
import org.szu.mnist.sul.DataClass;
import org.szu.mnist.sul.Model;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class MeanAlphaBeta extends AlphaBeta<MatrixSymbol> {

    public MeanAlphaBeta(Model model) {
        rowout = 4;
        colout = 4;
        rowspan = 4;
        colspan = 4;
        valuescale = 2;
        length = getLength();
        this.model = model;
    }

    @Override
    public Alphabet<MatrixSymbol> getAlphabet() {
        if(alphabet != null) return alphabet;
        List<MatrixSymbol> symbols = new ArrayList<>();
        for(int i = 0; i < this.valuescale; i++) {
            symbols.add(new MatrixSymbol(i));
        }
        alphabet = Alphabets.fromList(symbols);
        //alphabet = Alphabets.integers(0, this.valuescale-1);
        return alphabet;
    }

    /*
     ** mean, variance??
     */
    @Override
    protected MatrixSymbol toSymbol(INDArray data, int rowi, int coli){
        int value, total, inc;
        int rown, coln;
        float f;
        float[][] source = null;

        rown = Math.min(this.rowout, this.rowin-rowi);
        coln = Math.min(this.colout, this.colin-coli);
        if(this.saveSource) source = new float[rown][coln];
        value = 0; total = 0;
        for(int i = 0; i < rown; i++) {
            for(int j = 0; j < coln; j++) {
                //if(rowi+i >= this.rowin || coli+j >= this.colin) continue;
                f = data.getFloat((rowi + i) * this.colin + coli + j);
                if(this.saveSource) source[i][j] = f;
                inc = (int) (f * this.valuescale);
                if(inc >= this.valuescale) inc = this.valuescale - 1;
                value = value + inc;
                total++;
            }
        }
        if(total == 0) value = 0;
        else value = value / total;
        if(this.saveSource && value != 0) {
            if(!this.sources.containsKey(value)) {
                List<float[][]> list = new ArrayList<>();
                list.add(source);
                this.sources.put(value,list);
            } else {
                List<float[][]>  list = this.sources.get(value);
                //list.add(source);
                addList(list,source);
            }
        }
        return new MatrixSymbol(value);
    }


    @Override
    public INDArray toINDArray(Word<MatrixSymbol> word) {
        if(word.length() == 0) return null;
        if(this.withFormat &&  word.length() != length) return null;
        if(this.wordTrainSources!=null && this.wordTrainSources.containsKey(word)) {
            List<DataClass> list = this.wordTrainSources.get(word);
            List<DataClass> wordlist = new ArrayList<>();
            for(int i = 0; i <list.size(); i++) if(list.get(i).selected) wordlist.add(list.get(i));
            if(wordlist.size() > 0) {
                int selected = (int)(Math.random() * wordlist.size());
                return wordlist.get(selected).input;
            }
        }

        float[] featureData = new float[this.rowin * this.colin];
        int[] featureCounted = new int[this.rowin * this.colin];
        for(int i = 0; i < featureData.length; i++) {
            featureData[i] = 0.0f;
            featureCounted[i] = 0;
        }

        int rowi, coli;
        int index, length;
        int value;
        int roi, coi,rown, coln;

        rowi = 0; index = 0; length = word.length();
        if(length == 0)  return Nd4j.create(featureData);
        while(rowi  < this.rowin-this.rowout+1 && index < length) {
            coli = 0;
            while(coli < this.colin-this.colout+1 && index < length) {
                value = word.getSymbol(index).getValue();
                index++;
                if(this.sources.containsKey(value)) {
                    List<float[][]> list = this.sources.get(value);
                    int selected = (int)(Math.random() * list.size());
                    float[][] source = list.get(selected);
                    rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                    coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                    for (roi = 0; roi < rown; roi++) {
                        for (coi = 0; coi < coln; coi++) {
                           // if (rowi + roi < this.rowin && coli + coi < this.colin && roi < source.length && coi < source[0].length) {
                            featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                            featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                           // }
                        }
                    }
                }
                else {
                    rown = Math.min(this.rowout, this.rowin - rowi);
                    coln = Math.min(this.colout, this.colin - coli);
                    List<Integer> data = make4Mean(value, rown * coln);
                    for (roi = 0; roi < rown; roi++) {
                        for (coi = 0; coi < coln; coi++) {
                            //if (rowi + roi < this.rowin && coli + coi < this.colin && indexList < listSize) {//random.nextDouble()/2
                            featureData[(rowi + roi) * this.colin + (coli + coi)] += (data.get(roi*rown+coi) + Math.random()*delta) / this.valuescale;
                            featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                        }
                    }
                }
                coli = coli + this.colspan;
            }
            rowi = rowi+ this.rowspan;
        }

        for(int i = 0; i < featureData.length; i++) {
            if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
        }
        return Nd4j.create(featureData);
    }

    protected  List<List<float[][]>> getData(Word<MatrixSymbol> word) {
        int len, value, roi, coi;
        float[][] data;
        List<List<float[][]>> data4word = new ArrayList<>();
        len = word.length();
        for (int i = 0; i < len; i++) {
            value = word.getSymbol(i).getValue();
            if (this.sources.containsKey(value)) {
                data4word.add(this.sources.get(value));
                if(this.sources.get(value).size() <= 0) return null;
            }
            else {
                List<Integer> dataInt = make4Mean(value, this.rowout * this.colout);
                data = new float[this.rowout][this.colout];
                for (roi = 0; roi < this.rowout; roi++) {
                    for (coi = 0; coi < this.colout; coi++) {
                        data[roi][coi] =(float) (dataInt.get(roi * this.colout + coi) + Math.random() * delta) / this.valuescale;
                    }
                }
                List<float[][]> dataList = new ArrayList<>();
                dataList.add(data);
                data4word.add(dataList);
            }
        }
        return data4word;
    }

    @Override
    public List<INDArray> toINDArrayList(Word<MatrixSymbol> word) {
        if(word.length() == 0) return null;
        if(this.withFormat &&  word.length() != length) return null;
        if(this.wordTrainSources!=null && this.wordTrainSources.containsKey(word)) {
            List<DataClass> list = this.wordTrainSources.get(word);
            List<INDArray> wordlist = new ArrayList<>();
            for(int i = 0; i <list.size(); i++) if(list.get(i).selected) wordlist.add(list.get(i).input);
            if(wordlist.size() > this.listLimited) return wordlist.subList(0,this.listLimited-1);
            return wordlist;
        }


        List<List<float[][]>> data4word = getData(word);
        if(data4word == null) return null;
        int len = data4word.size();
        if(len <= 0) return null;
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for(int i = 0; i< len; i++) {
            dataIndexes[i] = 0;
            dataLens[i] = data4word.get(i).size();
        }

        List<INDArray> indArrayList = new ArrayList<>();
        float[] featureData;
        int[] featureCounted;
        int index,rowi,coli, rown, coln;
        while(true) {
            featureData = new float[this.rowin * this.colin];
            featureCounted = new int[this.rowin * this.colin];
            for(int i = 0; i < featureData.length; i++) {
                featureData[i] = 0.0f;
                featureCounted[i] = 0;
            }
            index = 0;  rowi = 0;
            while (rowi < this.rowin-this.rowout+1 && index < len) {
                coli = 0;
                while (coli < this.colin-this.colout+1 && index < len) {
                    float[][] source = data4word.get(index).get(dataIndexes[index]);
                    index++;
                    rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                    coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                    for (int roi = 0; roi < rown; roi++) {
                        for (int coi = 0; coi < coln; coi++) {
                            featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                            featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                        }
                    }
                    coli = coli + this.colspan;
                }
                rowi = rowi + this.rowspan;
            }
            for(int i = 0; i < featureData.length; i++) {
                if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
            }
            indArrayList.add(Nd4j.create(featureData));
            boolean done = true;
            for(int i = 0; i<len && done; i++) {
                if(dataIndexes[i] < dataLens[i] -1) done = false;
            }
            if(done || indArrayList.size() >= this.listLimited) break;
            int roi = 0;
            while(true) {
                dataIndexes[roi]++;
                if(dataIndexes[roi] >= dataLens[roi]) {
                    dataIndexes[roi] = 0;
                    roi++; continue;
                }
                break;
            }
        }
        return indArrayList;
    }

    private List<Integer> make4Mean (int avg, int total ) {
        List<Integer> result = new ArrayList<> ();
        int count = 1;
        int sum = avg * total;
        while (count <= total) {
            int rand = (int) ( Math.random () * sum );
            if (count < total) {
                result.add (rand);
            }
            else {
                result.add (sum - rand);
            }
            sum -= rand;
            count++;
        }
        return result;
    }

    @Override
    public List<MatrixSymbol> toSymbolList(INDArray data) {
        List<MatrixSymbol> symbols = new ArrayList<>();
        int rowi, coli;

        rowi = 0;
        while(rowi < this.rowin-this.rowout+1) {
            coli = 0;
            while (coli < this.colin-this.colout+1) {
                symbols.add(toSymbol(data,rowi,coli));
                coli += this.colspan;
            }
            rowi += this.rowspan;
        }
        return symbols;
    }

    @Override
    public BigInteger getAlphabetSize(){
        return BigInteger.valueOf(getAlphabet().size());
    }

    @Override
    public boolean query(Word<MatrixSymbol> word, boolean withWord) {
        if(word.length() == 0) return false;

        if(this.withFormat &&  word.length() != length) return false;

        if(wordTrainLabels.containsKey(word)) {
            return wordTrainLabels.get(word);
        }

        if(withWord) return false;

        List<List<float[][]>> data4word = getData(word);
        if(data4word == null) return false;
        int len = data4word.size();
        if(len <= 0) return false;
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for(int i = 0; i< len; i++) {
            dataIndexes[i] = 0;
            dataLens[i] = data4word.get(i).size();
        }

        float[] featureData;
        int[] featureCounted;
        int index, rowi, coli, rown, coln;
        int tNum, fNum;
        tNum = 0; fNum = 0;
        while(true) {
            featureData = new float[this.rowin * this.colin];
            featureCounted = new int[this.rowin * this.colin];
            for (int i = 0; i < featureData.length; i++) {
                featureData[i] = 0.0f;
                featureCounted[i] = 0;
            }
            index = 0;
            rowi = 0;
            while (rowi < this.rowin - this.rowout + 1 && index < len) {
                coli = 0;
                while (coli < this.colin - this.colout + 1 && index < len) {
                    float[][] source = data4word.get(index).get(dataIndexes[index]);
                    index++;
                    rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                    coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                    for (int roi = 0; roi < rown; roi++) {
                        for (int coi = 0; coi < coln; coi++) {
                            featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                            featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                        }
                    }
                    coli = coli + this.colspan;
                }
                rowi = rowi + this.rowspan;
            }
            for (int i = 0; i < featureData.length; i++) {
                if (featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
            }
            if(this.model.query(Nd4j.create(featureData))) tNum++;
            else fNum++;
            boolean done = true;
            for (int i = 0; i < len && done; i++) {
                if (dataIndexes[i] < dataLens[i] - 1) done = false;
            }
            if (done || (tNum+fNum) >= this.listLimited) break;
            int di = 0;
            while (true) {
                dataIndexes[di]++;
                if (dataIndexes[di] >= dataLens[di]) {
                    dataIndexes[di] = 0;
                    di++;
                    continue;
                }
                break;
            }
        }
        return tNum > fNum;
    }
}
