package org.szu.mnist.mapper;

import com.sun.tools.internal.xjc.reader.dtd.bindinfo.BIAttribute;
import com.sun.tools.internal.xjc.reader.xmlschema.bindinfo.BIGlobalBinding;
import net.automatalib.words.Alphabet;
import net.automatalib.words.Word;
import net.automatalib.words.impl.Alphabets;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.factory.Nd4j;
import org.szu.mnist.dfa.symbols.MatrixSymbol;
import org.szu.mnist.dfa.symbols.PositionSymbol;
import org.szu.mnist.sul.DataClass;
import org.szu.mnist.sul.Model;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

public class PositionMatrixAlphaBeta extends AlphaBeta<PositionSymbol> {

    private BigInteger symbolscale;

    public PositionMatrixAlphaBeta(Model model) {
        rowout = 2;
        colout = 2;
        rowspan = 2;
        colspan = 2;
        valuescale = 2;
        BigInteger valueTemp = BigInteger.valueOf(this.valuescale);
        symbolscale = valueTemp.pow(this.rowout * this.colout);
        length = getLength();
        this.model = model;
    }

    @Override
    public void setOutSize(int rowout, int colout) {
        super.setOutSize(rowout,colout);
        BigInteger valueTemp = BigInteger.valueOf(this.valuescale);
        symbolscale = valueTemp.pow(this.rowout * this.colout);
    }

    @Override
    public void setValueScale(int valuescale) {
        this.valuescale = valuescale;
        BigInteger valueTemp = BigInteger.valueOf(this.valuescale);
        symbolscale = valueTemp.pow(this.rowout * this.colout);
    }


    @Override
    public Alphabet<PositionSymbol> getAlphabet(){
        if(alphabet != null) return alphabet;

        List<PositionSymbol> symbols = new ArrayList<>();
        int rowi, coli;
        rowi = 0;
        while(rowi < this.rowin-this.rowout+1) {
            coli = 0;
            while (coli < this.colin-this.colout+1) {
                BigInteger temp = this.symbolscale;
                while(true) {
                    if(temp.equals(BigInteger.ONE)) break;
                    temp = temp.subtract(BigInteger.ONE);
                    symbols.add(new PositionSymbol(rowi, coli,temp));
                }
               // for(int i = 1; i < this.symbolscale; i++) //nonzero
                //    symbols.add(new PositionSymbol(rowi, coli, i));
                coli += this.colspan;
            }
            rowi += this.rowspan;
        }
        alphabet = Alphabets.fromList(symbols);
        return alphabet;
    }

    @Override
    public List<PositionSymbol> toSymbolList(INDArray data) {
        List<PositionSymbol> symbols = new ArrayList<>();
        int rowi, coli;
        PositionSymbol symbol;

        rowi = 0;
        while(rowi < this.rowin-this.rowout+1) {
            coli = 0;
            while (coli < this.colin-this.colout+1) {
                symbol = toSymbol(data,rowi,coli);
                if(!symbol.getValue_BIG().equals(BigInteger.ZERO)) symbols.add(symbol); //nonzero
                coli += this.colspan;
            }
            rowi += this.rowspan;
        }
        return symbols;
    }

    @Override
    protected PositionSymbol toSymbol(INDArray data, int rowi, int coli){
        BigInteger value, scale;
        int inc;
        int rown, coln;
        float f;
        float[][] source = null;

        rown = Math.min(this.rowout, this.rowin-rowi);
        coln = Math.min(this.colout, this.colin-coli);
        if(this.saveSource) source = new float[rown][coln];
        value = BigInteger.ZERO; scale = BigInteger.valueOf(this.valuescale);
        for(int i = 0; i < this.rowout; i++) {
            for(int j = 0; j < this.colout; j++) {
                if(rowi+i >= this.rowin || coli+j >= this.colin) value = value.multiply(scale);
                else {
                    f = data.getFloat((rowi + i) * this.colin + coli + j);
                    if (this.saveSource) source[i][j] = f;
                    inc = (int) (f * this.valuescale);
                    if (inc >= this.valuescale) inc = this.valuescale - 1;
                    value = value.multiply(scale).add(BigInteger.valueOf(inc));
                }
            }
        }
        if(this.saveSource && !value.equals(BigInteger.ZERO)) {
            if(!this.sources_BIG.containsKey(value)) {
                List<float[][]> list = new ArrayList<>();
                list.add(source);
                this.sources_BIG.put(value,list);
            } else {
                List<float[][]>  list = this.sources_BIG.get(value);
                //list.add(source);
                addList(list,source);
            }
        }
        return  new PositionSymbol(rowi, coli, value);
    }

    @Override
    public INDArray toINDArray(Word<PositionSymbol> word) {
        if(word.length() == 0) return null;
        if(this.wordTrainSources!=null && this.wordTrainSources.containsKey(word)) {
            List<DataClass> list = this.wordTrainSources.get(word);
            List<DataClass> wordlist = new ArrayList<>();
            for(int i = 0; i <list.size(); i++) if(list.get(i).selected) wordlist.add(list.get(i));
            if(wordlist.size() > 0) {
                int selected = (int)(Math.random() * wordlist.size());
                return wordlist.get(selected).input;
            }
        }

        float[] featureData = new float[this.rowin * this.colin];
        int[] featureCounted = new int[this.rowin * this.colin];
        for(int i = 0; i < featureData.length; i++) {
            featureData[i] = 0.0f;
            featureCounted[i] = 0;
        }

        try {
            int rowi, coli;
            int index, length;
            BigInteger value, scale;
            int roi, coi, i;
            PositionSymbol symbol;

            length = word.length();
            for(index = 0; index < length; index++) {
                symbol = word.getSymbol(index);
                rowi = symbol.getRowindex();
                coli = symbol.getColindex();
                value = symbol.getValue_BIG();
                if(this.sources_BIG.containsKey(value)) {
                    List<float[][]> list = this.sources_BIG.get(value);
                    int selected = (int)(Math.random() * list.size());
                    float[][] source = list.get(selected);
                    int rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                    int coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                    for (roi = 0; roi < rown; roi++) {
                        for (coi = 0; coi < coln; coi++) {
                            // if (rowi + roi < this.rowin && coli + coi < this.colin && roi < source.length && coi < source[0].length) {
                            featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                            featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                            // }
                        }
                    }
                } else {
                    roi = this.rowout - 1;
                    coi = this.colout - 1;
                    boolean done = false; scale = BigInteger.valueOf(this.valuescale);
                    while (value.compareTo(BigInteger.ZERO) > 0 && !done) {
                        if (rowi + roi < this.rowin && coli + coi < this.colin) {
                            i = (rowi + roi) * this.colin + (coli + coi);
                            featureData[i] += (float) ((value.remainder(scale).floatValue() + Math.random() * delta) / this.valuescale);
                            featureCounted[i]++;
                        }
                        value = value.divide(scale);
                        coi--;
                        if (coi < 0) {
                            coi = this.colout - 1;
                            roi--;
                            if(roi < 0) done = true;
                        }
                    }
                }
            }
        } catch (java.lang.Exception e) {
            e.printStackTrace();
        }
        for(int i = 0; i < featureData.length; i++) {
            if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
        }
        return Nd4j.create(featureData);
    }

    protected  List<List<float[][]>> getData(Word<PositionSymbol> word) {
        BigInteger value, scale;
        int len, roi,coi;
        boolean done;
        float[][] data;
        List<List<float[][]>> data4word = new ArrayList<>();
        len = word.length();
        for (int i = 0; i < len; i++) {
            value = word.getSymbol(i).getValue_BIG();
            if (this.sources_BIG.containsKey(value)) {
                data4word.add(this.sources_BIG.get(value));
                if(this.sources_BIG.get(value).size() <= 0) return null;
            }
            else {
                data = new float[this.rowout][this.colout];
                roi = this.rowout - 1;
                coi = this.colout - 1;
                done = false; scale = BigInteger.valueOf(this.valuescale);
                while (value.compareTo(BigInteger.ZERO) > 0 && !done) {
                    data[roi][coi] = (float) ((value.remainder(scale).floatValue() + Math.random() * delta) / this.valuescale);
                    value = value.divide(scale);
                    coi--;
                    if (coi < 0) {
                        coi = this.colout - 1;
                        roi--;
                        if (roi < 0) done = true;
                    }
                }
                List<float[][]> dataList = new ArrayList<>();
                dataList.add(data);
                data4word.add(dataList);
            }
        }
        return data4word;
    }

    @Override
    public List<INDArray> toINDArrayList(Word<PositionSymbol> word) {
        if(word.length() == 0) return null;
        if(this.withFormat && word.length() > length) return null;
        if(this.wordTrainSources != null && this.wordTrainSources.containsKey(word)) {
            List<DataClass> list = this.wordTrainSources.get(word);
            List<INDArray> wordlist = new ArrayList<>();
            for(int i = 0; i <list.size(); i++) if(list.get(i).selected) wordlist.add(list.get(i).input);
            if(wordlist.size() > this.listLimited) return wordlist.subList(0,this.listLimited-1);
            return wordlist;
        }

        List<List<float[][]>> data4word = getData(word);
        if(data4word == null) return null;
        int len = data4word.size();
        if(len <= 0) return null;
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for(int i = 0; i< len; i++) {
            dataIndexes[i] = 0;
            dataLens[i] = data4word.get(i).size();
        }

        List<INDArray> indArrayList = new ArrayList<>();
        float[] featureData;
        int[] featureCounted;
        int index,rowi,coli,rown,coln;
        while(true) {
            featureData = new float[this.rowin * this.colin];
            featureCounted = new int[this.rowin * this.colin];
            for(int i = 0; i < featureData.length; i++) {
                featureData[i] = 0.0f;
                featureCounted[i] = 0;
            }
            for(index = 0; index < len; index++) {
                rowi = word.getSymbol(index).getRowindex();
                coli = word.getSymbol(index).getColindex();
                float[][] source = data4word.get(index).get(dataIndexes[index]);
                index++;
                rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                for (int roi = 0; roi < rown; roi++) {
                    for (int coi = 0; coi < coln; coi++) {
                        featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                        featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                    }
                }
            }

            for(int i = 0; i < featureData.length; i++) {
                if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
            }
            indArrayList.add(Nd4j.create(featureData));

            boolean done = true;
            for(int i = 0; i<len && done; i++) {
                if(dataIndexes[i] < dataLens[i] -1) done = false;
            }
            if(done || indArrayList.size() >= this.listLimited) break;
            int roi = 0;
            while(true) {
                dataIndexes[roi]++;
                if(dataIndexes[roi] >= dataLens[roi]) {
                    dataIndexes[roi] = 0;
                    roi++; continue;
                }
                break;
            }
        }
        return indArrayList;
    }

    @Override
    public BigInteger getAlphabetSize(){
        return this.symbolscale.multiply(BigInteger.valueOf(this.length));
    }

    @Override
    public boolean query(Word<PositionSymbol> word, boolean withWord) {
        if(word.length() == 0) return false;
        if(this.withFormat && word.length() > length) return false;
        if(wordTrainLabels.containsKey(word)) {
            return wordTrainLabels.get(word);
        }
        if(withWord) return false;

        List<List<float[][]>> data4word = getData(word);
        if(data4word == null) return false;
        int len = data4word.size();
        if(len <= 0) return false;
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for(int i = 0; i< len; i++) {
            dataIndexes[i] = 0;
            dataLens[i] = data4word.get(i).size();
        }

        float[] featureData;
        int[] featureCounted;
        int index,rowi,coli,rown,coln;
        int tNum, fNum;
        tNum = 0; fNum = 0;
        while(true) {
            featureData = new float[this.rowin * this.colin];
            featureCounted = new int[this.rowin * this.colin];
            for(int i = 0; i < featureData.length; i++) {
                featureData[i] = 0.0f;
                featureCounted[i] = 0;
            }
            for(index = 0; index < len; index++) {
                rowi = word.getSymbol(index).getRowindex();
                coli = word.getSymbol(index).getColindex();
                float[][] source = data4word.get(index).get(dataIndexes[index]);
                index++;
                rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                for (int roi = 0; roi < rown; roi++) {
                    for (int coi = 0; coi < coln; coi++) {
                        featureData[(rowi + roi) * this.colin + (coli + coi)] += source[roi][coi];
                        featureCounted[(rowi + roi) * this.colin + (coli + coi)] += 1;
                    }
                }
            }

            for(int i = 0; i < featureData.length; i++) {
                if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
            }

            if(this.model.query(Nd4j.create(featureData))) tNum++;
            else fNum++;

            boolean done = true;
            for(int i = 0; i<len && done; i++) {
                if(dataIndexes[i] < dataLens[i] -1) done = false;
            }
            if(done || (tNum + fNum) >= this.listLimited) break;
            int roi = 0;
            while(true) {
                dataIndexes[roi]++;
                if(dataIndexes[roi] >= dataLens[roi]) {
                    dataIndexes[roi] = 0;
                    roi++; continue;
                }
                break;
            }
        }
        return tNum > fNum;
    }
}
