package org.szu.mnist.mapper;

import net.automatalib.words.Alphabet;
import net.automatalib.words.Word;
import net.automatalib.words.impl.Alphabets;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.factory.Nd4j;
import org.szu.mnist.dfa.symbols.LengthSymbol;
import org.szu.mnist.sul.DataClass;
import org.szu.mnist.sul.Model;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

public class RLESumAlphaBeta extends AlphaBeta<LengthSymbol> {


    public RLESumAlphaBeta(Model model) {
        rowout = 26;
        colout = 26;
        rowspan = 1;
        colspan = 1;
        valuescale = 2;
        length = getLength();
        this.model = model;
    }

    protected List<List<float[][]>> getData(Word<LengthSymbol> word) {
        int len, value;
        float[][] data;
        List<List<float[][]>> data4word = new ArrayList<>();
        len = word.length();
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for (int i = 0; i < len; i++) {
            value = word.getSymbol(i).getValue();
            if (this.sources.containsKey(value)) {
                data4word.add(this.sources.get(value));
                dataLens[i] = this.sources.get(value).size();
                if(dataLens[i] <= 0) return null;
                dataIndexes[i] = 0;
            }
            else {
                data = make4Sum(value,this.rowout,this.colout);
                List<float[][]> dataList = new ArrayList<>();
                dataList.add(data);
                data4word.add(dataList);
                dataLens[i] = 1;
                dataIndexes[i] = 0;
            }
        }
        return data4word;
    }

    @Override
    public List<INDArray> toINDArrayList(Word<LengthSymbol> word) {
        if(word.length() == 0) return null;
        if(this.withFormat && !lengthCheck(word)) return null;
        if(this.wordTrainSources!=null && this.wordTrainSources.containsKey(word)) {
            List<DataClass> list = this.wordTrainSources.get(word);
            List<INDArray> wordlist = new ArrayList<>();
            for(int i = 0; i <list.size(); i++) if(list.get(i).selected) wordlist.add(list.get(i).input);
            if(wordlist.size() > this.listLimited) return wordlist.subList(0,this.listLimited-1);
            return wordlist;
        }

        List<List<float[][]>> data4word = getData(word);
        if(data4word == null) return null;
        int len = data4word.size();
        if(len <= 0) return null;
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for(int i = 0; i< len; i++) {
            dataIndexes[i] = 0;
            dataLens[i] = data4word.get(i).size();
        }

        List<INDArray> indArrayList = new ArrayList<>();
        float[] featureData;
        int[] featureCounted;
        int index,rowi,coli,rown,coln,llen,l,ii;
        boolean done,done1;
        while(true) {
            featureData = new float[this.rowin * this.colin];
            featureCounted = new int[this.rowin * this.colin];
            for(int i = 0; i < featureData.length; i++) {
                featureData[i] = 0.0f;
                featureCounted[i] = 0;
            }

            rowi = 0; coli = 0; done = false;
            for(index = 0; index < len && !done; index++) {
                llen = word.getSymbol(index).getLen();
                for(l = 0; l < llen && !done; l++) {
                    float[][] source = data4word.get(index).get(dataIndexes[index]);
                    rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                    coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                    for (int roi = 0; roi < rown; roi++) {
                        for (int coi = 0; coi < coln; coi++) {
                            ii = (rowi + roi) * this.colin + (coli + coi);
                            featureData[ii] += source[roi][coi];
                            featureCounted[ii] += 1;
                        }
                    }
                    coli = coli + this.colspan;
                    if(coli >= this.colin-this.colout+1) {
                        coli = 0;
                        rowi = rowi + this.rowspan;
                        if(rowi >= this.rowin-this.rowout+1) done = true;
                    }
                }
            }
            for(int i = 0; i < featureData.length; i++) {
                if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
            }
            indArrayList.add(Nd4j.create(featureData));

            done1 = true;
            for(int i = 0; i<len && done1; i++) {
                if(dataIndexes[i] < dataLens[i] -1) done1 = false;
            }
            if(done1 || indArrayList.size() >= this.listLimited) break;
            int roi = 0;
            while(true) {
                dataIndexes[roi]++;
                if(dataIndexes[roi] >= dataLens[roi]) {
                    dataIndexes[roi] = 0;
                    roi++; continue;
                }
                break;
            }
        }
        return indArrayList;
    }

    @Override
    public boolean query(Word<LengthSymbol> word, boolean withWord) {
        if (word.length() == 0) return false;

        if (this.withFormat && !lengthCheck(word)) return false;

        if (wordTrainLabels.containsKey(word)) {
            return wordTrainLabels.get(word);
        }

        if (withWord) return false;

        List<List<float[][]>> data4word = getData(word);
        if (data4word == null) return false;
        int len = data4word.size();
        if (len <= 0) return false;
        int[] dataLens = new int[len];
        int[] dataIndexes = new int[len];
        for (int i = 0; i < len; i++) {
            dataIndexes[i] = 0;
            dataLens[i] = data4word.get(i).size();
        }

        float[] featureData;
        int[] featureCounted;
        int index,rowi,coli,rown,coln,llen,l,ii;
        boolean done,done1;
        int tNum,fNum;
        tNum = 0; fNum = 0;
        while(true) {
            featureData = new float[this.rowin * this.colin];
            featureCounted = new int[this.rowin * this.colin];
            for(int i = 0; i < featureData.length; i++) {
                featureData[i] = 0.0f;
                featureCounted[i] = 0;
            }

            rowi = 0; coli = 0; done = false;
            for(index = 0; index < len && !done; index++) {
                llen = word.getSymbol(index).getLen();
                for(l = 0; l < llen && !done; l++) {
                    float[][] source = data4word.get(index).get(dataIndexes[index]);
                    rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                    coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                    for (int roi = 0; roi < rown; roi++) {
                        for (int coi = 0; coi < coln; coi++) {
                            ii = (rowi + roi) * this.colin + (coli + coi);
                            featureData[ii] += source[roi][coi];
                            featureCounted[ii] += 1;
                        }
                    }
                    coli = coli + this.colspan;
                    if(coli >= this.colin-this.colout+1) {
                        coli = 0;
                        rowi = rowi + this.rowspan;
                        if(rowi >= this.rowin-this.rowout+1) done = true;
                    }
                }
            }
            for(int i = 0; i < featureData.length; i++) {
                if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
            }
            if(this.model.query(Nd4j.create(featureData))) tNum++;
            else fNum++;

            done1 = true;
            for(int i = 0; i<len && done1; i++) {
                if(dataIndexes[i] < dataLens[i] -1) done1 = false;
            }
            if(done1 || (tNum + fNum) >= this.listLimited) break;
            int roi = 0;
            while(true) {
                dataIndexes[roi]++;
                if(dataIndexes[roi] >= dataLens[roi]) {
                    dataIndexes[roi] = 0;
                    roi++; continue;
                }
                break;
            }
        }
        return tNum > fNum;
    }

    @Override
    public Alphabet<LengthSymbol> getAlphabet() {
        if(alphabet != null) return alphabet;

        List<LengthSymbol> symbols = new ArrayList<>();
        int total = this.rowout * this.colout * (this.valuescale - 1);
       // int rowi, coli;
       // rowi = 0;
       // while(rowi < this.rowin) {
        //    coli = 0;
          //  while (coli < this.colin) {
                for(int i = 0; i <= total; i++ ) {
                    for(int l = 1; l <= length; l++) {
                        symbols.add(new LengthSymbol(i, l));
                    }
                }
         //       coli += this.colspan;
         //   }
          //  rowi += this.rowspan;
       // }
        alphabet = Alphabets.fromList(symbols);
        return alphabet;
    }

    @Override
    public List<LengthSymbol> toSymbolList(INDArray data) {
        List<LengthSymbol> symbols = new ArrayList<>();
        int rowi, coli;
        LengthSymbol symbol;
        LengthSymbol lastsymbol = null;

        rowi = 0;
        while(rowi < this.rowin-this.rowout+1) {
            coli = 0;
            while (coli < this.colin-this.colout+1) {
                symbol = toSymbol(data,rowi,coli);
                if(lastsymbol != null && lastsymbol.getValue() == symbol.getValue()) {
                    lastsymbol.setLen(lastsymbol.getLen()+1);
                } else {
                    symbols.add(symbol);
                    lastsymbol = symbol;
                }
                coli += this.colspan;
            }
            rowi += this.rowspan;
        }
        return symbols;
    }

    @Override
    protected LengthSymbol toSymbol(INDArray data, int rowi, int coli){
        int value, inc;
        int rown, coln;
        float f;
        float[][] source =null;

        rown = Math.min(this.rowout, this.rowin-rowi);
        coln = Math.min(this.colout, this.colin-coli);
        if(this.saveSource) source = new float[rown][coln];
        value = 0;
        for(int i = 0; i < rown; i++) {
            for(int j = 0; j < coln; j++) {
                f = data.getFloat((rowi + i) * this.colin + coli + j);
                if(this.saveSource) source[i][j] = f;
                inc = (int) (f * this.valuescale);
                if(inc >= this.valuescale) inc = this.valuescale - 1;
                value = value + inc;
            }
        }
        if(this.saveSource && value != 0) {
            if(!this.sources.containsKey(value)) {
                List<float[][]> list = new ArrayList<>();
                list.add(source);
                this.sources.put(value,list);
            } else {
                List<float[][]>  list = this.sources.get(value);
                //list.add(source);
                addList(list,source);
            }
        }
        return new LengthSymbol(value,1);
    }

    private boolean lengthCheck(Word<LengthSymbol> word) {
        if(word == null) return false;
        int len = word.length();
        int total = 0;
        int value, lastvalue = -1;
        for(int i = 0; i<len; i++) {
            total += word.getSymbol(i).getLen();
            value = word.getSymbol(i).getValue();
            if(i > 0 && value == lastvalue) return false;
            lastvalue = value;
        }
        if(total == length) return true;
        return false;
    }

    @Override
    public INDArray toINDArray(Word<LengthSymbol> word) {
        if(this.withFormat && !lengthCheck(word)) return null;
        if(this.wordTrainSources!=null && this.wordTrainSources.containsKey(word)) {
            List<DataClass> list = this.wordTrainSources.get(word);
            List<DataClass> wordlist = new ArrayList<>();
            for(int i = 0; i <list.size(); i++) if(list.get(i).selected) wordlist.add(list.get(i));
            if(wordlist.size() > 0) {
                int selected = (int)(Math.random() * wordlist.size());
                return wordlist.get(selected).input;
            }
        }

        float[] featureData = new float[this.rowin * this.colin];
        int[] featureCounted = new int[this.rowin * this.colin];
        for(int i = 0; i < featureData.length; i++) {
            featureData[i] = 0.0f;
            featureCounted[i] = 0;
        }
        int rowi, coli;
        int index, length, len, l;
        int rown, coln, roi, coi;
        int value, i;
        LengthSymbol symbol;
        float[][] source;
        boolean done;

        length = word.length();
        if(length == 0)  return Nd4j.create(featureData);

        rowi = 0; coli = 0; done = false;
        for(index = 0; index < length && !done; index++) {
            symbol = word.getSymbol(index);
            value = symbol.getValue();
            len = symbol.getLen();
            for(l = 0; l < len && !done; l++) {
                if(this.sources.containsKey(value)) {
                    List<float[][]> list = this.sources.get(value);
                    int selected = (int)(Math.random() * list.size());
                    source = list.get(selected);
                    rown = Math.min(Math.min(this.rowout, this.rowin - rowi), source.length);
                    coln = Math.min(Math.min(this.colout, this.colin - coli), source[0].length);
                }
                else {
                    rown = Math.min(this.rowout, this.rowin - rowi);
                    coln = Math.min(this.colout, this.colin - coli);
                    source = make4Sum(value, rown, coln);
                }
                for (roi = 0; roi < rown; roi++) {
                    for (coi = 0; coi < coln; coi++) {
                        i = (rowi + roi) * this.colin + (coli + coi);
                        featureData[i] += source[roi][coi];
                        featureCounted[i] += 1;
                    }
                }
                coli = coli + this.colspan;
                if(coli >= this.colin-this.colout+1) {
                    coli = 0;
                    rowi = rowi + this.rowspan;
                    if(rowi >= this.rowin-this.rowout+1) done = true;
                }
            }
        }
        for(i = 0; i < featureData.length; i++) {
            if(featureCounted[i] > 1) featureData[i] = featureData[i] / featureCounted[i];
        }

        return Nd4j.create(featureData);
    }


    private float[][] make4Sum (int sum, int rown, int coln) {
        List<Integer> result = new ArrayList<> ();
        int num = rown * coln;
        int count = 1;
        while (count <= num) {
            int rand = (int) ( Math.random () * sum );
            if (count < num) {
                result.add (rand);
            }
            else {
                result.add (sum - rand);
            }
            sum -= rand;
            count++;
        }
        float [][] source = new float[rown][coln];
        for(int i = 0; i < rown; i++) {
            for(int j = 0; j < coln; j++) {
                source[i][j] = (float)((result.get(i*coln+j) +  Math.random()*delta) /this.valuescale);
            }
        }
        return source;
    }

    @Override
    public BigInteger getAlphabetSize(){
        return BigInteger.valueOf(getAlphabet().size());
    }
}
