package org.szu.mnist.sul;

import org.deeplearning4j.datasets.iterator.BaseDatasetIterator;
import org.deeplearning4j.datasets.iterator.impl.MnistDataSetIterator;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.dataset.api.DataSet;
import org.nd4j.linalg.dataset.api.iterator.DataSetIterator;
import org.nd4j.linalg.factory.Nd4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DataProvider {
    private static Logger log = LoggerFactory.getLogger(DataProvider.class);

    public static final int numRows = 28;
    public static final int numColumns = 28;
    public static final int outputNum = 2; // number of output classes
    private static  int numTrain = 60000;
    private static int numTest = 10000;
    private static int batchSize = 128; // batch size for each epoch
    private static int rngSeed = 123; // random number seed for reproducibility
    //private int numEpochs = 15; // numbebatchSizer of epochs to perform
    private static int kind = 1;

    private static DataSetIterator mnistTrain = null;
    private  static DataSetIterator mnistTest = null;

    private static DataSetIterator mnistTrainByKind = null;
    private  static DataSetIterator mnistTestByKind = null;

   /* public static INDArray ZERO = Nd4j.create(new double[]{1.0,0,0,0,0,0,0,0,0,0});
    public static INDArray ONE = Nd4j.create(new double[]{0,1.0,0,0,0,0,0,0,0,0});
    public static INDArray TWO = Nd4j.create(new double[]{0,0,1.0,0,0,0,0,0,0,0});
    public static INDArray THREE = Nd4j.create(new double[]{0,0,0,1.0,0,0,0,0,0,0});
    public static INDArray FOUR = Nd4j.create(new double[]{0,0,0,0,1.0,0,0,0,0,0});
    public static INDArray FIVE = Nd4j.create(new double[]{0,0,0,0,0,1.0,0,0,0,0});
    public static INDArray SIX = Nd4j.create(new double[]{0,0,0,0,0,0,1.0,0,0,0});
    public static INDArray SEVEN = Nd4j.create(new double[]{0,0,0,0,0,0,0,1.0,0,0});
    public static INDArray EIGHT = Nd4j.create(new double[]{0,0,0,0,0,0,0,0,1.0,0});
    public static INDArray NINE = Nd4j.create(new double[]{0,0,0,0,0,0,0,0,0,1.0});
    */


    private DataProvider() {

    }

    public static DataSetIterator getMnistTrain() throws Exception{
        if (mnistTrain == null) mnistTrain = new MnistDataSetIterator(batchSize, true, rngSeed);
        return mnistTrain;
    }

    public static DataSetIterator getMnistTest() throws Exception {
        if (mnistTest == null) mnistTest = new MnistDataSetIterator(batchSize, false, rngSeed);
        return mnistTest;
    }

    public static void setKind(int newkind) { //throws Exception
        if(kind == newkind) return;
        log.info("Set the num of pictures as " + newkind);
        kind = newkind;
        /*if (mnistTrain == null) mnistTrain = new MnistDataSetIterator(batchSize, true, rngSeed);
        mnistTrainByKind = selectDataByKind(mnistTrain, numTrain/batchSize);
        if (mnistTest == null) mnistTest = new MnistDataSetIterator(batchSize, false, rngSeed);
        mnistTestByKind = selectDataByKind(mnistTest, numTest/batchSize);*/
        mnistTrain = null;
        mnistTrainByKind = null;
        mnistTest = null;
        mnistTestByKind = null;
    }

    public static int getKind() {
        return kind;
    }

    public static int getBatchSize(){
        return batchSize;
    }

    public static int getTrainBatches() {
        return numTrain/batchSize;
    }

    public  static int getTestBatches() {
        return numTest/batchSize;
    }

    /*public static boolean compareResult(INDArray guess, INDArray label) {
        INDArray guessIndex = Nd4j.argMax(guess, 1);
        INDArray labelIndex = Nd4j.argMax(label, 1);
        if (guessIndex.getInt(0) == labelIndex.getInt(0)) return true;
        else return false;
    }*/

    public static DataSetIterator getMnistTrainByKind() throws Exception{
        if (mnistTrainByKind != null) return mnistTrainByKind;
        if (mnistTrain == null) mnistTrain = new MnistDataSetIterator(batchSize, true, rngSeed);
        mnistTrainByKind = selectDataByKind(mnistTrain, numTrain/batchSize);
        return mnistTrainByKind;
    }

    public static DataSetIterator getMnistTestByKind() throws Exception {
        if (mnistTestByKind != null) return mnistTestByKind;
        if (mnistTest == null) mnistTest = new MnistDataSetIterator(batchSize, false, rngSeed);
        mnistTestByKind = selectDataByKind(mnistTest, numTest/batchSize);
        return mnistTestByKind ;
    }

    //
    private static DataSetIterator selectDataByKind(DataSetIterator data, int batches) {
        log.info("Analyzing the data for binary classifier ......");
        DataSet d;
        INDArray labels, index;
        int length, t;
        float[][] newdata;
        MyDataFetcher fetcher;

        t = 0;
        fetcher = new MyDataFetcher();
        try {
            data.reset();
            while (t < batches) {
                d = data.next();
                labels = d.getLabels();
                length = labels.rows();
                newdata = new float[length][2];
                for (int i = 0; i < length; i++) {
                    index = Nd4j.argMax(labels.getRow(i), 1);
                    if (index.getInt(0) == kind) {
                        newdata[i][0] = 0.0f;
                        newdata[i][1] = 1.0f;
                    } else {
                        newdata[i][0] = 1.0f;
                        newdata[i][1] = 0.0f;
                    }
                }
                fetcher.addData(d.getFeatures(),Nd4j.create(newdata));
                t++;
            }
        } catch (java.lang.Exception e) {
            e.printStackTrace();
        }
        log.info("Proceeded: " + fetcher.totalExamples());
        data.reset();
        BaseDatasetIterator datasetIterator =  new BaseDatasetIterator(batchSize,fetcher.totalExamples(),fetcher);
        return datasetIterator;
    }

}
