package org.szu.mnist.dfa;

import net.automatalib.automata.fsa.impl.compact.CompactDFA;
import net.automatalib.automata.simple.SimpleAutomaton;
import net.automatalib.serialization.aut.AUTParser;
import net.automatalib.serialization.aut.AUTWriter;
import net.automatalib.serialization.taf.TAFSerializationDFA;
import net.automatalib.util.automata.builders.AutomatonBuilders;
import net.automatalib.words.Alphabet;
import net.automatalib.words.impl.Alphabets;
import org.apache.commons.io.FilenameUtils;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Collection;
import java.util.Iterator;


public class Test4SavingDFA {

    private static  String MNIST_ROOT = FilenameUtils.concat(System.getProperty("user.home"),"MNIST");
    private static  String dfafile = FilenameUtils.concat(MNIST_ROOT,"DFA");

    public static void main(String[] args) throws Exception {
        System.out.println("Hello, DFA for mnist！");
        CompactDFA<Character> dfa = constructSUL();
        //TAFSerializationDFA serializer = TAFSerializationDFA.getInstance();
       // serializer.writeModel(fos, dfa, dfa.getInputAlphabet());
        System.out.println("Before:");
        System.out.println("DFA:");
        System.out.println("\tAlphabet: " + dfa.getInputAlphabet().toString());
        System.out.println("\tStates: " + dfa.getStates().toString());
        System.out.println("\tInitial States: " + dfa.getInitialState().toString());
        System.out.println("\tFinal States:");
        Iterator<Integer> states = dfa.getStates().iterator();
        Integer state;
        System.out.print("\t\t");
        while(states.hasNext()) {
            state = states.next();
            System.out.print(" " + state + " : " + dfa.getStateProperty(state));
        }
        System.out.println();
        System.out.println("\tTrans: ");
        states = dfa.getStates().iterator();
        Character ch;
        while(states.hasNext()) {
            state = states.next();
            for(int i=0; i<dfa.getInputAlphabet().size();i++) {
                ch = dfa.getInputAlphabet().getSymbol(i);
                System.out.println("\t\t"+state + ", " + ch + " -> " + dfa.getTransition(state,ch));
            }
        }
        //System.out.println("\tFinal States: " + dfa.get.toString());
        FileOutputStream fos = new FileOutputStream(dfafile);
        AUTWriter.writeAutomaton(dfa,dfa.getInputAlphabet(),fos);
        fos.close();
        FileInputStream fis = new FileInputStream(dfafile);
        SimpleAutomaton<Integer, String> automaton = AUTParser.readAutomaton(fis).model;
        System.out.println("Before:");
        //System.out.println(automaton.getInputAlphabet().toString());
        System.out.println(automaton.getStates().toString());

    }

    private static CompactDFA<Character> constructSUL() {
        // input alphabet contains characters 'a'..'b'
        Alphabet<Character> sigma = Alphabets.characters('a', 'b');

        // @formatter:off
        // create automaton
        return AutomatonBuilders.newDFA(sigma)
            .withInitial("q0")
            .from("q0")
            .on('a').to("q1")
            .on('b').to("q2")
            .from("q1")
            .on('a').to("q0")
            .on('b').to("q3")
            .from("q2")
            .on('a').to("q3")
            .on('b').to("q0")
            .from("q3")
            .on('a').to("q2")
            .on('b').to("q1")
            .withAccepting("q0")
            .create();
        // @formatter:on
    }
}
